@@ -1,6 +1,6 @@
 ;;; navi2ch-board-misc.el --- Miscellaneous Functions for Navi2ch Board Mode
 
-;; Copyright (C) 2001, 2002 by Navi2ch Project
+;; Copyright (C) 2001-2003 by Navi2ch Project
 
 ;; Author: Taiki SUGAWARA <taiki@users.sourceforge.net>
 ;; Keywords: 2ch, network
@@ -23,10 +23,13 @@
 ;;; Code:
 (provide 'navi2ch-board-misc)
 (defvar navi2ch-board-misc-ident
-  "$Id: navi2ch-board-misc.el,v 1.46 2002/07/26 13:55:10 nanashi Exp $")
+  "$Id: navi2ch-board-misc.el,v 1.83 2003/09/12 13:07:30 mami Exp $")
 
 (eval-when-compile (require 'cl))
 
+;; Avoid byte-compile warnings (contrib/izonmoji-mode.el).
+(eval-when-compile (defvar izonmoji-mode nil))
+
 (require 'navi2ch)
 
 (defvar navi2ch-bm-mode-map nil)
@@ -52,6 +55,9 @@
     (define-key map "?" 'navi2ch-bm-search)
     (define-key map "\C-c\C-m" 'navi2ch-message-pop-message-buffer)
     (define-key map "R" 'navi2ch-bm-remove-article)
+    (define-key map "\C-c\C-r" 'navi2ch-bm-remove-article)
+    (define-key map "\C-o" 'navi2ch-bm-save-dat-file)
+    (define-key map "I" 'navi2ch-bm-fetch-maybe-new-articles)
 
     ;; mark command
     (define-key map "*" 'navi2ch-bm-mark)
@@ -77,6 +83,7 @@
 
 (defvar navi2ch-board-buffer-name "*navi2ch board*")
 
+;; set by navi2ch-bm-setup
 (defvar navi2ch-bm-get-property-function nil
   "$B$=$N0LCV$N(B text-property $B$rF@$k4X?t!#0z?t$O(B POINT")
 (defvar navi2ch-bm-set-property-function nil
@@ -88,6 +95,7 @@
 (defvar navi2ch-bm-exit-function nil)
 
 ;; stub functions
+;; set by navi2ch-bm-setup
 (defun navi2ch-bm-get-property-internal (point))
 (defun navi2ch-bm-set-property-internal (begin end item))
 (defun navi2ch-bm-get-board-internal (item))
@@ -100,27 +108,33 @@
 (defvar navi2ch-bm-state-alist
   '((view "V"
 	  navi2ch-bm-view-face
+	  navi2ch-bm-new-view-face
 	  navi2ch-bm-updated-view-face
 	  navi2ch-bm-seen-view-face)
     (cache "C"
 	   navi2ch-bm-cache-face
+	   navi2ch-bm-new-cache-face
 	   navi2ch-bm-updated-cache-face
 	   navi2ch-bm-seen-cache-face)
     (update "U"
 	    navi2ch-bm-update-face
+	    navi2ch-bm-new-update-face
 	    navi2ch-bm-updated-update-face
 	    navi2ch-bm-seen-update-face)
     (nil " "
 	 navi2ch-bm-unread-face
+	 navi2ch-bm-new-unread-face
 	 navi2ch-bm-updated-unread-face
 	 navi2ch-bm-seen-unread-face)
     (mark " "
 	  navi2ch-bm-mark-face
+	  navi2ch-bm-new-mark-face
 	  navi2ch-bm-updated-mark-face
 	  navi2ch-bm-seen-mark-face)))
 
 (defvar navi2ch-bm-updated-mark-alist
-  '((updated . "+")
+  '((new . "%")
+    (updated . "+")
     (seen . "=")
     (nil . " ")))
 
@@ -167,14 +181,15 @@
 	(navi2ch-put-alist type open-func
 			   navi2ch-bm-board-type-alist))
   (when board
-    (setq navi2ch-list-navi2ch-category-alist
-	  (cons board navi2ch-list-navi2ch-category-alist))))
+    (add-to-list 'navi2ch-list-navi2ch-category-alist board)))
 
 (defun navi2ch-bm-select-board (board &optional force)
-  (switch-to-buffer (get-buffer-create navi2ch-board-buffer-name))
-  (let ((type (cdr (assq 'type board))))
+  (let ((buf (get-buffer-create navi2ch-board-buffer-name))
+	(type (cdr (assq 'type board))))
+    (set-buffer buf)
     (funcall (cdr (assq type navi2ch-bm-board-type-alist))
-	     board force))
+	     board force)
+    (switch-to-buffer buf))
   (run-hooks 'navi2ch-bm-select-board-hook)
   (navi2ch-set-mode-line-identification))
 
@@ -183,9 +198,10 @@
   (setq updated (or updated (get-text-property (1+ begin) 'updated)))
   (put-text-property begin end 'updated updated)
   (put-text-property begin end 'mouse-face navi2ch-bm-mouse-face)
-  (put-text-property begin end 'face (nth (cond ((eq updated 'updated) 3)
-						((eq updated 'seen) 4)
-						((eq updated nil) 2))
+  (put-text-property begin end 'face (nth (cond ((null updated) 2)
+						((eq updated 'new) 3)
+						((eq updated 'updated) 4)
+						((eq updated 'seen) 5))
 					  (assq state
 						navi2ch-bm-state-alist))))
 
@@ -210,16 +226,34 @@
 		      (t
 		       (navi2ch-article-check-cached board article)))))
     (unless subject (setq subject navi2ch-bm-empty-subject))
-    (insert (format "%3d %s%s %s%s%s\n"
-                    number
-		    (cdr (assq updated navi2ch-bm-updated-mark-alist))
-		    (cadr (assq state navi2ch-bm-state-alist))
-                    subject
-                    (make-string (max (- navi2ch-bm-subject-width
-                                         (string-width subject))
-                                      1)
-                                 ? )
-                    other))
+    ;; for contrib/izonmoji-mode.el
+    (navi2ch-ifxemacs
+	(insert (format (concat "%" (number-to-string navi2ch-bm-number-width)
+				"d %s%s %s%s%s\n")
+			number
+			(cdr (assq updated navi2ch-bm-updated-mark-alist))
+			(cadr (assq state navi2ch-bm-state-alist))
+			subject
+			(make-string (max (- navi2ch-bm-subject-width
+					     (string-width subject))
+					  1)
+				     ? )
+			other))
+      (let ((buffer-display-table (if (and (boundp 'izonmoji-mode)
+					   izonmoji-mode)
+				      nil
+				    buffer-display-table)))
+	(insert (format (concat "%" (number-to-string navi2ch-bm-number-width)
+				"d %s%s %s%s%s\n")
+			number
+			(cdr (assq updated navi2ch-bm-updated-mark-alist))
+			(cadr (assq state navi2ch-bm-state-alist))
+			subject
+			(make-string (max (- navi2ch-bm-subject-width
+					     (string-width subject))
+					  1)
+				     ? )
+			other))))
     (navi2ch-bm-set-property point (1- (point)) item state updated)))
 
 (defun navi2ch-bm-exit ()
@@ -238,20 +272,46 @@
 	  (select-window win)
 	(navi2ch-list)))))
 
-(defun navi2ch-bm-goto-state-column ()
+;;; goto-*-column
+(defun navi2ch-bm-goto-updated-mark-column ()
   (beginning-of-line)
-  (forward-char 5))
+  (when (looking-at " *[0-9]+ ")
+    (goto-char (match-end 0))))
+
+(defun navi2ch-bm-goto-state-column ()
+  (when (navi2ch-bm-goto-updated-mark-column)
+    (forward-char 1)))
+
+(defun navi2ch-bm-goto-mark-column ()
+  (when (navi2ch-bm-goto-updated-mark-column)
+    (forward-char 2)))
+
+(defun navi2ch-bm-goto-other-column ()
+  (let ((sbj (cdr
+              (assq 'subject
+		    (navi2ch-bm-get-article-internal
+                     (navi2ch-bm-get-property-internal (point)))))))
+    (navi2ch-bm-goto-mark-column)
+    (forward-char 1)
+    (unless sbj (setq sbj navi2ch-bm-empty-subject))
+    (when (and (not (string= sbj ""))
+               (search-forward sbj nil t))
+      (goto-char (match-end 0)))
+    (skip-chars-forward " ")))
+
 
 (defun navi2ch-bm-insert-state (item state &optional updated)
   ;; (setq article (navi2ch-put-alist 'cache 'view article))
-  (navi2ch-bm-goto-state-column)
-  (backward-char 1)
-  (delete-char 2)
-  (insert (cdr (assq updated navi2ch-bm-updated-mark-alist)))
-  (insert (cadr (assq state navi2ch-bm-state-alist)))
-  (navi2ch-bm-set-property (save-excursion (beginning-of-line) (point))
-			   (save-excursion (end-of-line) (point))
-			   item state updated))
+  (let ((buffer-read-only nil))
+    (save-excursion
+      (navi2ch-bm-goto-state-column)
+      (backward-char 1)
+      (delete-char 2)
+      (insert (cdr (assq updated navi2ch-bm-updated-mark-alist)))
+      (insert (cadr (assq state navi2ch-bm-state-alist)))
+      (navi2ch-bm-set-property (navi2ch-line-beginning-position)
+			       (navi2ch-line-end-position)
+			       item state updated))))
 
 (defun navi2ch-bm-get-state (&optional point)
   "$B$=$N0LCV$N(B state $B$rD4$Y$k(B"
@@ -263,6 +323,14 @@
 			  (cons (cadr x) (car x)))
 			navi2ch-bm-state-alist)))))
 
+(defun navi2ch-bm-get-updated-mark (&optional point)
+  "$B$=$N0LCV$N(B updated-mark $B$rD4$Y$k(B"
+  (save-excursion
+    (and point (goto-char point))
+    (when (navi2ch-bm-goto-updated-mark-column)
+      (car (rassoc (char-to-string (char-after))
+		   navi2ch-bm-updated-mark-alist)))))
+
 (defun navi2ch-bm-select-article (&optional max-line)
   (interactive "P")
   (let* ((item (navi2ch-bm-get-property-internal (point)))
@@ -283,38 +351,39 @@
 			 board article nil nil max-line)))
 		(save-excursion
 		  (set-buffer buf)
-		  (let ((buffer-read-only nil))
-		    (when (or state
-			      (navi2ch-bm-fetched-article-p board article)
-			      (eq (navi2ch-bm-get-state) 'view))
-		      (navi2ch-bm-remove-fetched-article board article)
-		      (if (eq major-mode 'navi2ch-board-mode)
-			  (navi2ch-bm-insert-state item 'view 'seen)
-			(navi2ch-bm-insert-state item 'view)))))))
+		  (when (or state
+			    (navi2ch-bm-fetched-article-p board article)
+			    (eq (navi2ch-bm-get-state) 'view))
+		    (navi2ch-bm-remove-fetched-article board article)
+		    (if (eq major-mode 'navi2ch-board-mode)
+			(navi2ch-bm-insert-state item 'view 'seen)
+		      (navi2ch-bm-insert-state item 'view))))))
 	  (message "Can't select this line!"))
       ((error quit)
        (set-window-configuration window-configuration)
        (signal (car err) (cdr err))))))
 
 (defun navi2ch-bm-show-url ()
-  "$BHD$N(Burl $B$rI=<($7$F!"$=$N(B url $B$r8+$k$+(B kill ring $B$K%3%T!<$9$k(B"
+  "$BHD$N(B url $B$rI=<($7$F!"$=$N(B url $B$r8+$k$+(B kill ring $B$K%3%T!<$9$k(B"
   (interactive)
   (let* ((board (navi2ch-bm-get-board-internal
 		 (navi2ch-bm-get-property-internal (point))))
 	 (url (navi2ch-board-to-url board)))
-    (let ((char (navi2ch-read-char-with-retry
-		 (format "c)opy v)iew t)itle? URL: %s: " url)
-		 nil '(?c ?v ?t))))
-      (if (eq char ?t) (navi2ch-bm-copy-title board)
-	(funcall (cond ((eq char ?c)
-			(lambda (x)
-			  (kill-new x)
-			  (message "copy: %s" x)))
-		       ((eq char ?v)
-			(lambda (x)
-			  (navi2ch-browse-url-internal x)
-			  (message "view: %s" x))))
-		 (navi2ch-bm-show-url-subr board))))))
+    (if (not url)
+	(message "Can't select this line!")
+      (let ((char (navi2ch-read-char-with-retry
+		   (format "c)opy v)iew t)itle? URL: %s: " url)
+		   nil '(?c ?v ?t))))
+	(if (eq char ?t) (navi2ch-bm-copy-title board)
+	  (funcall (cond ((eq char ?c)
+			  (lambda (x)
+			    (kill-new x)
+			    (message "copy: %s" x)))
+			 ((eq char ?v)
+			  (lambda (x)
+			    (navi2ch-browse-url-internal x)
+			    (message "view: %s" x))))
+		   (navi2ch-bm-show-url-subr board)))))))
 
 (defun navi2ch-bm-show-url-subr (board)
   "$B%a%K%e!<$rI=<($7$F!"(Burl $B$rF@$k(B"
@@ -372,14 +441,43 @@
          state)
     (if (and article
 	     (not (navi2ch-board-from-file-p board)))
-	(progn
+	(let (summary artid element seen)
+	  (when (and navi2ch-board-check-article-update-suppression-length
+		     (not (navi2ch-bm-fetched-article-p board article)))
+	    (setq summary (navi2ch-article-load-article-summary board))
+	    (setq artid (cdr (assq 'artid article)))
+	    (setq element (cdr (assoc artid summary)))
+	    (setq seen (or (navi2ch-article-summary-element-seen element)
+			   (cdr (assoc artid navi2ch-board-last-seen-alist))
+			   0)))
 	  (setq state (navi2ch-article-fetch-article board article force))
 	  (when state
-	    (navi2ch-bm-remember-fetched-article board article)
-	    (let ((buffer-read-only nil))
-	      (save-excursion
-		(navi2ch-bm-insert-state item 'update)))))
-      (message "Can't select this line!"))))
+	    (let ((state-mark 'update)
+		  (updated-mark (navi2ch-bm-get-updated-mark)))
+	      (when seen
+		(setq seen
+		      (and (catch 'break
+			     (<= (string-to-number
+				  (or (cdr (assoc artid navi2ch-board-subject-alist))
+				      (throw 'break t)))
+				 (+ seen navi2ch-board-check-article-update-suppression-length)))
+			   (navi2ch-article-check-message-suppression
+			    board
+			    article
+			    (1+ seen)
+			    (+ seen navi2ch-board-check-article-update-suppression-length)))))
+	      (if seen
+		  (progn
+		    (navi2ch-article-summary-element-set-seen element seen)
+		    (navi2ch-article-save-article-summary board summary)
+		    (setq state-mark (navi2ch-bm-get-state))
+		    (when (memq updated-mark '(new updated))
+		      (setq updated-mark 'seen))
+		    (message "No updates need seeing"))
+		(navi2ch-bm-remember-fetched-article board article))
+	      (navi2ch-bm-insert-state item state-mark updated-mark))))
+      (message "Can't select this line!"))
+    state))
 
 (defun navi2ch-bm-textize-article (&optional dir-or-file buffer)
   (interactive)
@@ -392,33 +490,9 @@
     (when navi2ch-article-view-range
       (setq navi2ch-article-view-range nil)
       (navi2ch-article-redraw))
-    (let* ((article navi2ch-article-current-article)
-	   (board navi2ch-article-current-board)
-	   (id (cdr (assq 'id board)))
-	   (subject (cdr (assq 'subject article)))
-	   (basename (format "%s_%s.txt" id (cdr (assq 'artid article))))
-	   (dir nil) file)
-      (and dir-or-file
-	   (file-directory-p dir-or-file)
-	   (setq dir dir-or-file))
-      (setq file
-	    (if (or (not dir-or-file)
-		    (and dir (interactive-p)))
-		(expand-file-name
-		 (read-file-name "Write thread to file: " dir nil nil basename))
-	      (expand-file-name basename dir)))
-      (and buffer
-	   (save-excursion
-	     (set-buffer buffer)
-	     (goto-char (point-max))
-	     (insert (format "<a href=\"%s\">%s</a><br>\n" file subject))))
-      (let ((coding-system-for-write navi2ch-coding-system))
-	(navi2ch-write-region (point-min) (point-max)
-			      file))
-      (message "Wrote %s" file))
+    (navi2ch-article-textize-article dir-or-file buffer)
     (select-window window)))
 
-
 (defun navi2ch-bm-select-article-or-scroll (way &optional max-line)
   (let ((article (navi2ch-bm-get-article-internal
 		  (navi2ch-bm-get-property-internal (point)))))
@@ -469,10 +543,17 @@
     (let ((buffer-read-only nil)
           (i 1))
       (while (not (eobp))
-        (let ((props (text-properties-at (point))))
-          (delete-char 3)
-          (insert (format "%3d" i))
-	  (set-text-properties (- (point) 3) (point) props)
+        (let ((props (text-properties-at (point)))
+	      (num-string (format
+			   (concat "%" (number-to-string navi2ch-bm-number-width) "d")
+			   i)))
+          (delete-region (point)
+			 (save-excursion
+			   (navi2ch-bm-goto-state-column)
+			   (- (point) 2)))
+          (insert num-string)
+	  (set-text-properties (- (point) (length num-string))
+			       (point) props)
           (forward-line 1)
           (setq i (1+ i)))))))
 
@@ -488,8 +569,9 @@
     (setq old-file (cdr (assq 'logo board)))
     (if navi2ch-offline
 	(setq file old-file)
-      (setq file (file-name-nondirectory (navi2ch-net-download-logo board)))
+      (setq file (navi2ch-net-download-logo board))
       (when file
+	(setq file (file-name-nondirectory (navi2ch-net-download-logo board)))
 	(when (and old-file navi2ch-board-delete-old-logo
 		   (not (string-equal file old-file)))
 	  (delete-file (navi2ch-board-get-file-name board old-file)))
@@ -516,21 +598,27 @@
       (message "Can't select this line!"))))
 
 ;;; move
-(defun navi2ch-bm-next-line ()
-  (interactive)
-  (forward-line 1)
+(defun navi2ch-bm-forward-line (&optional n)
+  (interactive "p")
+  (let ((ret (forward-line n)))
+    (when (eobp)
+      (forward-line -1)
+      (setq ret (1+ ret)))
+    ret))
+
+(defun navi2ch-bm-next-line (num)
+  (interactive "p")
+  (unless (zerop (navi2ch-bm-forward-line num))
+    (message "No more articles"))
   (setq navi2ch-bm-move-downward t))
 
-(defun navi2ch-bm-previous-line ()
-  (interactive)
-  (forward-line -1)
+(defun navi2ch-bm-previous-line (num)
+  (interactive "p")
+  (unless (zerop (navi2ch-bm-forward-line (- num)))
+    (message "No more articles"))
   (setq navi2ch-bm-move-downward nil))
 
 ;;; mark
-(defun navi2ch-bm-goto-mark-column ()
-  (navi2ch-bm-goto-state-column)
-  (forward-char 1))
-
 (defun navi2ch-bm-mark-subr (mark &optional arg interactive)
   "mark $B$9$k!#(B
 INTERACTIVE $B$,(B non-nil $B$J$i(B mark $B$7$?$"$H0\F0$9$k!#(B
@@ -563,9 +651,7 @@ ARG $B$,(B non-nil $B$J$i0\F0J}8~$r5U$K$9$k!#(B"
 		       navi2ch-bm-move-downward)))
 	      ((eq navi2ch-bm-mark-and-move t)
 	       (setq downward (not arg))))
-	(if downward
-	    (navi2ch-bm-next-line)
-	  (navi2ch-bm-previous-line))))))
+	(navi2ch-bm-forward-line (if downward 1 -1))))))
 
 (defun navi2ch-bm-mark (&optional arg)
   (interactive "P")
@@ -633,26 +719,37 @@ ARG $B$,(B non-nil $B$J$i0\F0J}8~$r5U$K$9$k!#(B"
   (navi2ch-bm-mark-region-subr (save-excursion (goto-char begin)
 					       (beginning-of-line)
 					       (point))
-			       (save-excursion (goto-char end)
-					       (backward-char 1)
+			       (save-excursion (goto-char (max (1- end)
+							       (point-min)))
 					       (end-of-line)
 					       (point))
 			       (if arg " " "*")))
 
+(defun navi2ch-bm-fetch-maybe-new-articles ()
+  "$B99?7$5$l$F$$$k2DG=@-$N$"$k%9%l$r(B fetch $B$9$k!#(B"
+  (interactive)
+  (navi2ch-bm-mark-states "[^=]")
+  (sit-for 0)
+  (navi2ch-bookmark-fetch-mark-article))
+
 (defun navi2ch-bm-mark-all (&optional arg)
   (interactive "P")
   (navi2ch-bm-mark-region (point-min) (point-max) arg))
 
 (defun navi2ch-bm-mark-marks (mark &optional arg)
-  (interactive "cinput mark: \nP")
+  (interactive "cInput mark: \nP")
+  (navi2ch-bm-mark-states
+   (format ".%c" (upcase mark))
+   arg))
+
+(defun navi2ch-bm-mark-states (regexp &optional arg)
   (save-excursion
     (goto-char (point-min))
-    (let ((rep (format "%c" (upcase mark))))
-      (while (not (eobp))
-        (navi2ch-bm-goto-state-column)
-        (when (looking-at rep)
-          (navi2ch-bm-mark-subr (if arg " " "*")))
-        (forward-line)))))
+    (while (not (eobp))
+      (navi2ch-bm-goto-updated-mark-column)
+      (when (looking-at regexp)
+	(navi2ch-bm-mark-subr (if arg " " "*")))
+      (forward-line))))
 
 ;; mark by regexp query
 (defun navi2ch-bm-mark-by-query (query &optional arg)
@@ -670,34 +767,27 @@ ARG $B$,(B non-nil $B$J$i0\F0J}8~$r5U$K$9$k!#(B"
       (sort-subr rev 'forward-line 'end-of-line
                  start-key-fun end-key-fun))))
 
-(defun navi2ch-bm-sort-by-number (rev)
+(defun navi2ch-bm-sort-by-number (&optional rev)
   (interactive "P")
   (navi2ch-bm-sort-subr
    rev
    'beginning-of-line
    'navi2ch-bm-goto-state-column))
 
-(defun navi2ch-bm-sort-by-state (rev)
+(defun navi2ch-bm-sort-by-state (&optional rev)
   (interactive "P")
   (navi2ch-bm-sort-subr
-   (not rev)
-   'navi2ch-bm-goto-state-column
-   'forward-char))
+   rev
+   (lambda ()
+     (navi2ch-bm-goto-state-column)
+     (backward-char)
+     (or (cdr (assoc (buffer-substring (point) (+ (point) 2))
+		     navi2ch-bm-sort-by-state-order))
+	 ;; $BL$CN$N>uBV!#(B
+	 1000))
+   nil))
 
-(defun navi2ch-bm-goto-other-column ()
-  (let ((sbj (cdr
-              (assq 'subject
-		    (navi2ch-bm-get-article-internal
-                     (navi2ch-bm-get-property-internal (point)))))))
-    (navi2ch-bm-goto-mark-column)
-    (forward-char 1)
-    (unless sbj (setq sbj navi2ch-bm-empty-subject))
-    (when (and (not (string= sbj ""))
-               (search-forward sbj nil t))
-      (goto-char (match-end 0)))
-    (skip-chars-forward " ")))
-
-(defun navi2ch-bm-sort-by-subject (rev)
+(defun navi2ch-bm-sort-by-subject (&optional rev)
   (interactive "P")
   (navi2ch-bm-sort-subr
    rev
@@ -706,7 +796,7 @@ ARG $B$,(B non-nil $B$J$i0\F0J}8~$r5U$K$9$k!#(B"
      (forward-char 1))
    'navi2ch-bm-goto-other-column))
 
-(defun navi2ch-bm-sort-by-other (rev)
+(defun navi2ch-bm-sort-by-other (&optional rev)
   (interactive "P")
   (navi2ch-bm-sort-subr
    rev
@@ -715,7 +805,7 @@ ARG $B$,(B non-nil $B$J$i0\F0J}8~$r5U$K$9$k!#(B"
      nil) ; end-key-fun $B$r8F$P$;$k$K$O(B nil $B$,M_$7$$$i$7$$!#$O$^$C$?(B($B5c(B)$B!#(B
    'end-of-line))
 
-(defun navi2ch-bm-sort-by-date (rev)
+(defun navi2ch-bm-sort-by-date (&optional rev)
   (interactive "P")
   (navi2ch-bm-sort-subr
    (not rev)
@@ -731,13 +821,15 @@ ARG $B$,(B non-nil $B$J$i0\F0J}8~$r5U$K$9$k!#(B"
   (let ((ch (navi2ch-read-char-with-retry
 	     "Sort by n)umber s)tate t)itle o)ther d)ate? "
 	     nil '(?n ?s ?t ?o ?d))))
+    (message "Sorting...")
     (funcall
      (cond ((eq ch ?n) 'navi2ch-bm-sort-by-number)
            ((eq ch ?s) 'navi2ch-bm-sort-by-state)
            ((eq ch ?t) 'navi2ch-bm-sort-by-subject)
            ((eq ch ?o) 'navi2ch-bm-sort-by-other)
 	   ((eq ch ?d) 'navi2ch-bm-sort-by-date))
-     arg)))
+     arg)
+    (message "Sorting...done")))
 
 ;;; search
 (defun navi2ch-bm-search-current-board-subject ()
@@ -776,11 +868,9 @@ ARG $B$,(B non-nil $B$J$i0\F0J}8~$r5U$K$9$k!#(B"
 
 ;;; save and load info
 (defun navi2ch-bm-save-info ()
-  (if navi2ch-bm-fetched-article-list
-      (navi2ch-save-info
-       navi2ch-bm-fetched-info-file
-       navi2ch-bm-fetched-article-list
-       t)))
+  (navi2ch-save-info navi2ch-bm-fetched-info-file
+		     navi2ch-bm-fetched-article-list
+		     t))
 
 (defun navi2ch-bm-load-info ()
   (setq navi2ch-bm-fetched-article-list
@@ -793,30 +883,29 @@ ARTILCES $B$,(B alist $B$N>l9g$O$=$N%9%l$N$_$r!"(Balist $B$N(B list $B$N>
   (let ((summary (navi2ch-article-load-article-summary board)))
     (setq articles
 	  (cond ((cdr (assq 'artid articles)) ; $B%9%l(B alist
-		 (setq articles (list articles)))
+		 (list articles))
 		((cdr (assq 'artid (car articles))) ; $B%9%l(B alist $B$N(B list
-		 articles)
-		(t nil)))
+		 articles)))
     (dolist (article articles)
-      (let ((artid (cdr (assq 'artid article))))
-	(let ((buffer (get-buffer (navi2ch-article-get-buffer-name board
-								   article))))
-	  (when buffer
-	    (delete-windows-on buffer)
-	    (kill-buffer buffer)))
+      (let ((artid (cdr (assq 'artid article)))
+	    (buffer (get-buffer (navi2ch-article-get-buffer-name board
+								 article)))
+	    elt)
+	(when buffer
+	  (delete-windows-on buffer)
+	  (kill-buffer buffer))
 	(dolist (file (list (navi2ch-article-get-info-file-name board article)
-			    (navi2ch-article-get-file-name board article)))
+			    (navi2ch-article-get-file-name board article)
+			    (navi2ch-article-get-message-filter-cache-file-name
+			     board article)))
 	  (condition-case nil
 	      (if (file-exists-p file)
 		  (delete-file file))
 	    (error nil)))
 	(navi2ch-bm-remove-fetched-article board article)
-	(when summary
-	  (let (elt)
-	    (while (setq elt (assoc artid summary)) ; $B%/%I$$(B?
-	      (setq summary (delq elt summary)))))))
-    (when summary
-      (navi2ch-article-save-article-summary board summary))))
+	(while (setq elt (assoc artid summary)) ; $B%/%I$$(B?
+	  (setq summary (delq elt summary)))))
+    (navi2ch-article-save-article-summary board summary)))
 
 (defun navi2ch-bm-remove-article ()
   (interactive)
@@ -825,13 +914,19 @@ ARTILCES $B$,(B alist $B$N>l9g$O$=$N%9%l$N$_$r!"(Balist $B$N(B list $B$N>
 	 (board (navi2ch-bm-get-board-internal item)))
     (when (and board article)
       (navi2ch-bm-remove-article-subr board article)
-      (save-excursion
-	(let ((buffer-read-only nil))
-	  (navi2ch-bm-insert-state item nil nil))))))
+      (navi2ch-bm-insert-state item nil nil))))
 
 (defun navi2ch-bm-remove-mark-article ()
   (interactive)
   (navi2ch-bm-exec-subr 'navi2ch-bm-remove-article))
 
+(defun navi2ch-bm-save-dat-file ()
+  (interactive)
+  (let* ((item (navi2ch-bm-get-property-internal (point)))
+	 (article (navi2ch-bm-get-article-internal item))
+	 (board (navi2ch-bm-get-board-internal item)))
+    (when (and board article)
+      (navi2ch-article-save-dat-file board article))))
+
 (run-hooks 'navi2ch-board-misc-load-hook)
 ;;; navi2ch-board-misc.el ends here
