@@ -1,6 +1,6 @@
 ;;; navi2ch-bookmark.el --- global bookmark module for navi2ch
 
-;; Copyright (C) 2001, 2002 by Navi2ch Project
+;; Copyright (C) 2001-2003 by Navi2ch Project
 
 ;; Author: Taiki SUGAWARA <taiki@users.sourceforge.net>
 ;; Keywords: network, 2ch
@@ -27,7 +27,7 @@
 ;;; Code:
 (provide 'navi2ch-bookmark)
 (defvar navi2ch-bookmark-ident
-  "$Id: navi2ch-bookmark.el,v 1.34 2002/10/02 15:44:16 mami Exp $")
+  "$Id: navi2ch-bookmark.el,v 1.37 2003/07/16 12:30:48 mami Exp $")
 
 (eval-when-compile (require 'cl))
 
@@ -360,6 +360,7 @@ KEY $B$O(B (concat URI ARTID) ")
   (setq major-mode 'navi2ch-bookmark-mode)
   (setq mode-name "Navi2ch Bookmark")
   (setq buffer-read-only t)
+  (buffer-disable-undo)
   (use-local-map navi2ch-bookmark-mode-map)
   (navi2ch-bookmark-setup-menu)
   (run-hooks 'navi2ch-bm-mode-hook 'navi2ch-bookmark-mode-hook))
@@ -384,8 +385,7 @@ KEY $B$O(B (concat URI ARTID) ")
 						 (assq 'artid article)))))))
 			  (cddr y))))
 		      navi2ch-bookmark-list)))
-    (if info
-	(navi2ch-save-info navi2ch-bookmark-file info t))))
+    (navi2ch-save-info navi2ch-bookmark-file info t)))
 
 (defun navi2ch-bookmark-load-info ()
   (setq navi2ch-bookmark-list
