@@ -30,7 +30,7 @@
 ;;; Code:
 (provide 'navi2ch-jbbs-net)
 (defvar navi2ch-jbbs-net-ident
-  "$Id: navi2ch-jbbs-net.el,v 1.12 2002/10/17 15:22:23 mami Exp $")
+  "$Id: navi2ch-jbbs-net.el,v 1.13 2003/09/07 12:26:16 mami Exp $")
 
 (require 'navi2ch-multibbs)
 
@@ -91,8 +91,12 @@ START, END, NOFIRST $B$GHO0O$r;XDj$9$k(B"
     (concat
      (format "%s/bbs/read.cgi?BBS=%s&KEY=%s"
 	     (match-string 1 uri) (match-string 2 uri) artid)
-     (and start (format "&START=%d" start))
-     (and end (format "&END=%d" end))
+     (if (and (stringp start)
+	      (string-match "l\\([0-9]+\\)" start))
+	 (format "&LAST=%s" (match-string 1 start))
+       (concat
+	(and start (format "&START=%d" start))
+	(and end (format "&END=%d" end))))
      (and nofirst
 	  (not (eq start 1))
 	  "&NOFIRST=TRUE"))))
