@@ -30,7 +30,7 @@
 ;;; Code:
 (provide 'navi2ch-jbbs-shitaraba)
 (defvar navi2ch-jbbs-shitaraba-ident
-  "$Id: navi2ch-jbbs-shitaraba.el,v 1.18 2002/10/17 15:22:23 mami Exp $")
+  "$Id: navi2ch-jbbs-shitaraba.el,v 1.20 2003/09/07 12:26:16 mami Exp $")
 
 (eval-when-compile (require 'cl))
 (require 'navi2ch-util)
@@ -99,14 +99,18 @@ START $B$,(B non-nil $B$J$i$P%l%9HV9f(B START $B$+$i$N:9J,$r<hF@$9$k!#(B
 (defun navi2ch-js-url-to-article (url)
   "URL $B$+$i(B article $B$KJQ49!#(B"
   (let (list)
-    (if (string-match
-	 "http://[^/]+/[^/]+/bbs/read\\.cgi.*KEY=\\([0-9]+\\)" url)
-	(progn
-	  (setq list (list (cons 'artid (match-string 1 url))))
-	  (when (string-match "&START=\\([0-9]+\\)" url)
-	    (setq list (cons (cons 'number
-				   (string-to-number (match-string 1 url)))
-			     list)))))
+    (cond ((string-match
+	    "http://[^/]+/[^/]+/bbs/read\\.cgi.*KEY=\\([0-9]+\\)" url)
+	   (setq list (list (cons 'artid (match-string 1 url))))
+	   (when (string-match "&START=\\([0-9]+\\)" url)
+	     (setq list (cons (cons 'number
+				    (string-to-number (match-string 1 url)))
+			      list))))
+	  ;; "http://jbbs.shitaraba.com/computer/351/storage/1022028671.html" $B$H$+!#(B
+	  ((string-match
+	    "http://.+/storage/\\([0-9]+\\)\\.html" url)
+	   (setq list (list (cons 'artid (match-string 1 url))
+			    (cons 'kako t)))))
     list))
 
 (defun navi2ch-js-send-message
@@ -152,8 +156,12 @@ START, END, NOFIRST $B$GHO0O$r;XDj$9$k(B"
     (concat
      (format "%s/bbs/read.cgi?BBS=%s&KEY=%s"
 	     (match-string 1 uri) (match-string 2 uri) artid)
-     (and start (format "&START=%d" start))
-     (and end (format "&END=%d" end))
+     (if (and (stringp start)
+	      (string-match "l\\([0-9]+\\)" start))
+	 (format "&LAST=%s" (match-string 1 start))
+       (concat
+	(and start (format "&START=%d" start))
+	(and end (format "&END=%d" end))))
      (and nofirst
 	  (not (eq start 1))
 	  "&NOFIRST=TRUE"))))
