@@ -1,6 +1,6 @@
 ;;; navi2ch-list.el --- board list module for navi2ch
 
-;; Copyright (C) 2000-2002 by Navi2ch Project
+;; Copyright (C) 2000-2003 by Navi2ch Project
 
 ;; Author: Taiki SUGAWARA <taiki@users.sourceforge.net>
 ;; Keywords: network, 2ch
@@ -27,7 +27,7 @@
 ;;; Code:
 (provide 'navi2ch-list)
 (defvar navi2ch-list-ident
-  "$Id: navi2ch-list.el,v 1.44 2002/07/26 13:55:11 nanashi Exp $")
+  "$Id: navi2ch-list.el,v 1.61 2003/07/16 12:30:48 mami Exp $")
 
 (eval-when-compile (require 'cl))
 
@@ -73,6 +73,7 @@
 (defvar navi2ch-list-buffer-name "*navi2ch list*")
 (defvar navi2ch-list-current-list nil)
 (defvar navi2ch-list-category-list nil)
+(defvar navi2ch-list-board-name-list nil)
 
 (defvar navi2ch-list-navi2ch-category-name "Navi2ch")
 (defvar navi2ch-list-changed-category-name "$BJQ$o$C$?HD(B")
@@ -84,6 +85,9 @@
     (change "C" navi2ch-list-change-board-name-face)
     (nil " " navi2ch-list-board-name-face)))
 
+(defconst navi2ch-list-bbstable-default-url
+  "http://www.ff.iij4u.or.jp/~ch2/bbsmenu.html")
+
 ;; add hook
 (add-hook 'navi2ch-save-status-hook 'navi2ch-list-save-info)
 
@@ -116,7 +120,9 @@
     (setcdr category
 	    (list
 	     (cadr category)
-	     (cons 'child list)))))
+	     (cons 'child list))))
+  (setq navi2ch-list-board-name-list (navi2ch-list-get-board-name-list
+				      navi2ch-list-category-list)))
 
 (defun navi2ch-list-get-global-bookmark-board-list ()
   (mapcar (lambda (x)
@@ -162,6 +168,8 @@
   (when (file-exists-p file)
     (with-temp-buffer
       (navi2ch-insert-file-contents file)
+      (run-hooks 'navi2ch-list-get-category-list-hook)
+      (navi2ch-apply-filters navi2ch-list-current-list navi2ch-list-filter-list)
       (goto-char (point-min))
       (let (list)
 	(while (re-search-forward "\\(.+\\)\n\n\n" nil t)
@@ -205,8 +213,12 @@
 	(insert (car state)
 		indent
 		(cdr (assq 'name board)) "\n")
+	(set-text-properties prev (point) nil)
 	(set-text-properties
-	 prev (1- (point))
+	 (+ prev
+	    (length (car state))
+	    (length indent))
+	 (1- (point))
 	 (list 'mouse-face navi2ch-list-mouse-face
 	       'face (cadr state))))
       (put-text-property prev (point) 'board board)
@@ -225,7 +237,7 @@
 	  (set-text-properties prev (1- (point))
 			       (list 'mouse-face navi2ch-list-mouse-face
 				     'face 'navi2ch-list-category-face))
-	  (put-text-property prev (point) 'category (car pair))
+	  (put-text-property prev (point) 'genre (car pair))
 	  (when open
 	    (navi2ch-list-insert-board-names-subr (cdr (assq 'child alist))))
 	  (setq prev (point)))))))
@@ -258,28 +270,28 @@
 (defun navi2ch-list-toggle-open ()
   "$B%+%F%4%j$r3+$$$?$jJD$8$?$j$9$k!#(B"
   (interactive)
-  (save-excursion
-    (let (category props)
-      (end-of-line)
-      (when (re-search-backward "^\\[[+-]\\]" nil t)
-	(setq category (get-text-property (point) 'category))
-	(setq props (text-properties-at (point)))
-	(let* ((pair (assoc category navi2ch-list-category-list))
-	       (alist (cdr pair))
-	       (open (cdr (assq 'open alist)))
-	       (buffer-read-only nil))
-	  (delete-region (point) (+ 3 (point)))
-	  (insert "[" (if (not open) "-" "+") "]")
-	  (set-text-properties (- (point) 3) (point) props)
-	  (forward-line 1)
-	  (if open
-	      (let ((begin (point))
-		    (end (point-max)))
-		(when (re-search-forward "^\\[[+-]\\]" nil t)
-		  (setq end (match-beginning 0)))
-		(delete-region begin end))
-	    (navi2ch-list-insert-board-names-subr (cdr (assq 'child alist))))
-	  (setcdr pair (navi2ch-put-alist 'open (not open) alist)))))))
+  (when (save-excursion
+	  (end-of-line)
+	  (re-search-backward "^\\[[+-]\\]" nil t))
+    (goto-char (match-beginning 0))
+    (let* ((category (get-text-property (point) 'genre))
+	   (props (text-properties-at (point)))
+	   (pair (assoc category navi2ch-list-category-list))
+	   (alist (cdr pair))
+	   (open (cdr (assq 'open alist)))
+	   (buffer-read-only nil))
+      (delete-region (point) (+ 3 (point)))
+      (insert "[" (if open "+" "-") "]")
+      (set-text-properties (- (point) 3) (point) props)
+      (save-excursion
+	(forward-line 1)
+	(if open
+	    (delete-region (point)
+			   (if (re-search-forward "^\\[[+-]\\]" nil t)
+			       (match-beginning 0)
+			     (point-max)))
+	  (navi2ch-list-insert-board-names-subr (cdr (assq 'child alist)))))
+      (setcdr pair (navi2ch-put-alist 'open (not open) alist)))))
 
 (defun navi2ch-list-select-current-board (&optional force)
   "$BHD$rA*$V!#$^$?$O%+%F%4%j$N3+JD$r$9$k(B"
@@ -287,47 +299,62 @@
   (let (prop)
     (cond ((setq prop (get-text-property (point) 'board))
 	   (navi2ch-list-select-board prop force))
-	  ((get-text-property (point) 'category)
+	  ((get-text-property (point) 'genre)
 	   (navi2ch-list-toggle-open))
 	  (t
 	   (message "can't select this line!")))))
 
 (defun navi2ch-list-open-all-category ()
   (interactive)
-  (let ((str (buffer-substring-no-properties
-	      (save-excursion (beginning-of-line)
-			      (+ navi2ch-list-indent-width (point)))
-	      (save-excursion (end-of-line) (point)))))
-    (setq navi2ch-list-category-list
-	  (mapcar (lambda (x)
-		    (navi2ch-put-alist 'open t x))
-		  navi2ch-list-category-list))
-    (let ((buffer-read-only nil))
-      (erase-buffer)
-      (navi2ch-list-insert-board-names
-       navi2ch-list-category-list))
-    (goto-char (point-min))
-    (search-forward str nil t)))
+  (when (save-excursion
+	  (end-of-line)
+	  (re-search-backward "^\\[[+-]\\]" nil t))
+    (let ((str (buffer-substring-no-properties
+		(save-excursion (beginning-of-line) (point))
+		(save-excursion (end-of-line) (point)))))
+      (setq navi2ch-list-category-list
+	    (mapcar (lambda (x)
+		      (navi2ch-put-alist 'open t x))
+		    navi2ch-list-category-list))
+      (let ((buffer-read-only nil))
+	(erase-buffer)
+	(navi2ch-list-insert-board-names
+	 navi2ch-list-category-list))
+      (goto-char (point-min))
+      (re-search-forward (concat "^"
+				 (regexp-quote
+				  (navi2ch-replace-string "^\\[\\+\\]" "[-]"
+							  str t))
+				 "$")
+			 nil t)
+      (beginning-of-line)
+      (if (looking-at "\\[-\\]")
+	  (goto-char (match-end 0))
+	(forward-char navi2ch-list-indent-width)))))
 
 (defun navi2ch-list-close-all-category ()
   (interactive)
-  (let (str)
-    (end-of-line)
-    (re-search-backward "^\\[[+-]\\]" nil t)
-    (setq str (buffer-substring-no-properties
-	       (save-excursion (beginning-of-line)
-			       (+ navi2ch-list-indent-width (point)))
-	       (save-excursion (end-of-line) (point))))
-    (setq navi2ch-list-category-list
-	  (mapcar (lambda (x)
-		    (navi2ch-put-alist 'open nil x))
-		  navi2ch-list-category-list))
-    (let ((buffer-read-only nil))
-      (erase-buffer)
-      (navi2ch-list-insert-board-names
-       navi2ch-list-category-list))
-    (goto-char (point-min))
-    (search-forward str nil t)))
+  (when (save-excursion
+	  (end-of-line)
+	  (re-search-backward "^\\[[+-]\\]" nil t))
+    (goto-char (match-end 0))
+    (let ((str (buffer-substring-no-properties
+		(point)
+		(save-excursion (end-of-line) (point)))))
+      (setq navi2ch-list-category-list
+	    (mapcar (lambda (x)
+		      (navi2ch-put-alist 'open nil x))
+		    navi2ch-list-category-list))
+      (let ((buffer-read-only nil))
+	(erase-buffer)
+	(navi2ch-list-insert-board-names
+	 navi2ch-list-category-list))
+      (goto-char (point-min))
+      (re-search-forward (concat "^\\(\\[[+-]\\]\\)"
+				 (regexp-quote str)
+				 "$")
+			 nil t)
+      (goto-char (match-end 1)))))
 
 (defun navi2ch-list-select-board (board &optional force)
   (let ((flag (eq (current-buffer)
@@ -352,9 +379,11 @@
 (defun navi2ch-list-mode ()
   "\\{navi2ch-list-mode-map}"
   (interactive)
+  (kill-all-local-variables)
   (setq major-mode 'navi2ch-list-mode)
   (setq mode-name "Navi2ch List")
   (setq buffer-read-only t)
+  (buffer-disable-undo)
   (use-local-map navi2ch-list-mode-map)
   (navi2ch-list-setup-menu)
   (run-hooks 'navi2ch-list-mode-hook)
@@ -413,6 +442,22 @@ changed-list $B$O(B '((board-id old-board new-board) ...) $B$J(B alist$B!#
     (list (cons 'add added-list)
  	  (cons 'change changed-list))))
 
+(defun navi2ch-list-change (changed-list)
+  "CHANGED-LIST $B$r$b$H$KHD%V%C%/%^!<%/$r99?7(B"
+  (let ((changed-alist (mapcar
+			(lambda (elt)
+			  (cons (navi2ch-list-bookmark-node (nth 1 elt))
+				(navi2ch-list-bookmark-node (nth 2 elt))))
+			changed-list)))
+    (setq navi2ch-list-current-list
+	  (navi2ch-put-alist 'bookmark
+			     (mapcar (lambda (node)
+				       (or (cdr (assoc node changed-alist))
+					   node))
+				     (cdr (assq 'bookmark
+						navi2ch-list-current-list)))
+			     navi2ch-list-current-list))))
+
 (defun navi2ch-list-apply-changed-status (changed-status)
   "CHANGED-STATUS $B$r$b$H$KHD$NJQ99$r$$$m$s$J=j$KH?1G$9$k!#(B"
   (message "applying board changes...")
@@ -431,6 +476,7 @@ changed-list $B$O(B '((board-id old-board new-board) ...) $B$J(B alist$B!#
     (navi2ch-change-log-directory changed-list)
     (navi2ch-bookmark-change changed-list)
     (navi2ch-history-change changed-list)
+    (navi2ch-list-change changed-list)
     (message "applying board changes...done"))))
 
 (defun navi2ch-list-get-changed-category (category-list)
@@ -450,41 +496,59 @@ changed-list $B$O(B '((board-id old-board new-board) ...) $B$J(B alist$B!#
 	  (navi2ch-net-force-update (or navi2ch-net-force-update
 					force))
 	  (file (navi2ch-list-get-file-name))
-	  updated old-category-list)
-      (setq old-category-list
-	    (if first
-		(progn
-		  (navi2ch-list-load-info)
-		  (navi2ch-list-get-category-list file))
-	      (navi2ch-list-get-normal-category-list
-	       navi2ch-list-category-list)))
+	  (bbstable (or navi2ch-list-bbstable-url
+			navi2ch-list-bbstable-default-url))
+	  (change (cdr (assq 'change navi2ch-list-current-list)))
+	  updated header time old-category-list)
+      (when first
+	(navi2ch-list-load-info))
+      (navi2ch-set-mode-line-identification)
+      (setq old-category-list (navi2ch-list-get-normal-category-list
+			       navi2ch-list-category-list))
       (unless (or navi2ch-offline
 		  (and first
 		       (not navi2ch-list-sync-update-on-boot)
 		       (file-exists-p file)))
-	(setq updated (navi2ch-net-update-file
-		       navi2ch-list-bbstable-url file nil
-		       'navi2ch-list-make-board-txt)))
-      (when t				;(or first updated)
-	(erase-buffer)
-	(let ((category-list (navi2ch-list-get-category-list file)))
-	  (when updated
-	    (navi2ch-list-apply-changed-status
-	     (navi2ch-list-get-changed-status
-	      old-category-list category-list)))
-	  (setq navi2ch-list-category-list
-		(append
-		 (delq nil
-		       (list (navi2ch-list-get-category
-			      navi2ch-list-navi2ch-category-name
-			      navi2ch-list-navi2ch-category-alist)
-			     (navi2ch-list-get-global-bookmark-category)
-			     (navi2ch-list-get-etc-category)
-			     (navi2ch-list-get-changed-category
-			      category-list)))
-		 category-list)))
-	(navi2ch-set-mode-line-identification)
-	(navi2ch-list-insert-board-names navi2ch-list-category-list))))
+	(setq time (and (equal (cdr (assq 'bbstable navi2ch-list-current-list))
+			       bbstable)
+			(cdr (assq 'time navi2ch-list-current-list))))
+	(setq header (navi2ch-net-update-file bbstable file time
+					      'navi2ch-list-make-board-txt))
+	(setq updated (and header
+			   (not (navi2ch-net-get-state 'not-updated header))
+			   (not (navi2ch-net-get-state 'error header)))))
+      (when updated
+	(setq navi2ch-list-current-list
+	      (navi2ch-put-alist 'time
+				 (or (cdr (assoc "Last-Modified" header))
+				     (cdr (assoc "Date" header)))
+				 navi2ch-list-current-list))
+	(setq navi2ch-list-current-list
+	      (navi2ch-put-alist 'bbstable bbstable
+				 navi2ch-list-current-list)))
+      ;; bbstable, etc.txt, navi2ch-list-navi2ch-category-alist
+      ;; $B$N$$$:$l$+$,99?7$5$l$F$$$l$P(B $B0J2<$N=hM}$,I,MW!#(B
+      ;; $B$H$j$"$($:!">o$K<B9T$7$F$*$/!#(B
+      (let ((category-list (navi2ch-list-get-category-list file)))
+	(when (or updated change)
+	  (navi2ch-list-apply-changed-status
+	   (navi2ch-list-get-changed-status old-category-list category-list)))
+	(setq navi2ch-list-category-list
+	      (append
+	       (delq nil
+		     (list (navi2ch-list-get-category
+			    navi2ch-list-navi2ch-category-name
+			    navi2ch-list-navi2ch-category-alist)
+			   (navi2ch-list-get-global-bookmark-category)
+			   (navi2ch-list-get-etc-category)
+			   (navi2ch-list-get-changed-category
+			    category-list)))
+	       category-list))
+	(setq navi2ch-list-board-name-list
+	      (navi2ch-list-get-board-name-list
+	       navi2ch-list-category-list)))
+      (erase-buffer)
+      (navi2ch-list-insert-board-names navi2ch-list-category-list)))
   (run-hooks 'navi2ch-list-after-sync-hook))
 
 (defun navi2ch-list-board-id-from-url (url)
@@ -558,10 +622,8 @@ changed-list $B$O(B '((board-id old-board new-board) ...) $B$J(B alist$B!#
 (defun navi2ch-list-goto-board (&optional default)
   (interactive)
   (let (alist board)
-    (setq alist (mapcar
-		 (lambda (x) (cons (cdr (assq 'id x)) x))
-		 (navi2ch-list-get-board-name-list
-		  navi2ch-list-category-list)))
+    (setq alist (mapcar (lambda (x) (cons (cdr (assq 'id x)) x))
+			navi2ch-list-board-name-list))
     (save-window-excursion
       (setq board (cdr (assoc
 			(completing-read
@@ -597,6 +659,15 @@ changed-list $B$O(B '((board-id old-board new-board) ...) $B$J(B alist$B!#
 			    (delq (assoc id alist) alist))))))
     (mapcar #'cdr (nreverse alist))))
 
+(defun navi2ch-list-normalize-bookmark (list)
+  (let ((bookmark (cdr (assq 'bookmark list)))
+	ret)
+    (dolist (x (navi2ch-list-get-board-name-list navi2ch-list-category-list))
+      (let ((node (navi2ch-list-bookmark-node x)))
+	(when (member node bookmark)
+	  (setq ret (cons node (delq node ret))))))
+    (nreverse ret)))
+
 (defun navi2ch-list-save-info ()
   (when navi2ch-list-category-list
     (let ((list (mapcar (lambda (elt)
@@ -609,9 +680,12 @@ changed-list $B$O(B '((board-id old-board new-board) ...) $B$J(B alist$B!#
   (when navi2ch-list-current-list
     (navi2ch-save-info
      (navi2ch-list-get-file-name "list.info")
-     (list (assq 'bookmark navi2ch-list-current-list)
+     (list (cons 'bookmark (navi2ch-list-normalize-bookmark
+			    navi2ch-list-current-list))
 	   (assq 'category navi2ch-list-current-list)
-	   (assq 'change navi2ch-list-current-list))
+	   (assq 'change navi2ch-list-current-list)
+	   (assq 'bbstable navi2ch-list-current-list)
+	   (assq 'time navi2ch-list-current-list))
      t)))
 
 (defun navi2ch-list-load-info ()
@@ -619,13 +693,27 @@ changed-list $B$O(B '((board-id old-board new-board) ...) $B$J(B alist$B!#
 	(navi2ch-load-info (navi2ch-list-get-file-name "list.info")))
   (if navi2ch-list-load-category-list
       (setq navi2ch-list-category-list
-	    (cdr (assq 'category navi2ch-list-current-list)))))
+	    (cdr (assq 'category navi2ch-list-current-list))))
+  (let* ((file (navi2ch-list-get-file-name))
+	 (category-list (navi2ch-list-get-category-list file)))
+    (setq navi2ch-list-category-list
+	  (append
+	   (delq nil
+		 (list (navi2ch-list-get-category
+			navi2ch-list-navi2ch-category-name
+			navi2ch-list-navi2ch-category-alist)
+		       (navi2ch-list-get-global-bookmark-category)
+		       (navi2ch-list-get-etc-category)
+		       (navi2ch-list-get-changed-category category-list)))
+	   category-list))
+    (setq navi2ch-list-board-name-list
+	  (navi2ch-list-get-board-name-list navi2ch-list-category-list))))
 
 (defun navi2ch-list-get-current-category-list ()
   (save-excursion
     (end-of-line)
     (when (re-search-backward "^\\[[+-]\\]" nil t)
-      (let ((category (get-text-property (point) 'category)))
+      (let ((category (get-text-property (point) 'genre)))
 	(cdr (assq 'child (cdr
 			   (assoc category
 				  navi2ch-list-category-list))))))))
@@ -714,8 +802,11 @@ changed-list $B$O(B '((board-id old-board new-board) ...) $B$J(B alist$B!#
   (interactive)
   (let ((ch (navi2ch-read-char-with-retry
 	     "Search for: s)ubject a)rticle c)ache: " nil '(?s ?a ?c)))
-	(ch2 (navi2ch-read-char-with-retry
-	      "Search from: b)oard c)ategory a)ll: " nil '(?b ?c ?a))))
+	(ch2 (if (get-text-property (point) 'board)
+		 (navi2ch-read-char-with-retry
+		  "Search from: b)oard c)ategory a)ll: " nil '(?b ?c ?a))
+	       (navi2ch-read-char-with-retry
+		"Search from: c)ategory a)ll: " nil '(?c ?a)))))
     (cond ((eq ch ?s)
 	   (cond ((eq ch2 ?b) (navi2ch-list-search-current-board-subject))
 		 ((eq ch2 ?c) (navi2ch-list-search-current-category-subject))
@@ -749,8 +840,7 @@ changed-list $B$O(B '((board-id old-board new-board) ...) $B$J(B alist$B!#
   (and (interactive-p) (setq ask t))
   (when (or (not ask)
 	    (y-or-n-p "Expire all boards?"))
-    (dolist (board (navi2ch-list-get-board-name-list
-		    navi2ch-list-category-list))
+    (dolist (board navi2ch-list-board-name-list)
       (when (eq (cdr (assq 'type board)) 'board)
 	(navi2ch-board-expire board)))
     (message "expiring all board is done")))
@@ -768,21 +858,23 @@ changed-list $B$O(B '((board-id old-board new-board) ...) $B$J(B alist$B!#
   (let* ((board (get-text-property (point) 'board))
 	 (uri (cdr (assq 'uri board)))
 	 (name (cdr (assq 'name board))))
-    (let ((char (navi2ch-read-char-with-retry
-		 (format "c)opy v)iew t)itle u)rl&title? URL: %s: " uri)
-		 nil '(?c ?v ?t ?u))))
-      (if (eq char ?v)
-	  (navi2ch-browse-url-internal uri)
-	(let ((str (cond ((eq char ?c)
-			  uri)
-			 ((eq char ?t)
-			  name)
-			 ((eq char ?u)
-			  (format "%s\n%s" name uri)))))
-	  (if (not str)
-	      (ding)
-	    (kill-new str)
-	    (message "copy: %s" str)))))))
+    (if (not uri)
+	(message "Can't select this line!")
+      (let ((char (navi2ch-read-char-with-retry
+		   (format "c)opy v)iew t)itle u)rl&title? URL: %s: " uri)
+		   nil '(?c ?v ?t ?u))))
+	(if (eq char ?v)
+	    (navi2ch-browse-url-internal uri)
+	  (let ((str (cond ((eq char ?c)
+			    uri)
+			   ((eq char ?t)
+			    name)
+			   ((eq char ?u)
+			    (format "%s\n%s" name uri)))))
+	    (if (not str)
+		(ding)
+	      (kill-new str)
+	      (message "copy: %s" str))))))))
 
 (run-hooks 'navi2ch-list-load-hook)
 ;;; navi2ch-list.el ends here
