@@ -1,6 +1,6 @@
 ;;; navi2ch-message.el --- write message module for navi2ch
 
-;; Copyright (C) 2000-2002 by Navi2ch Project
+;; Copyright (C) 2000-2003 by Navi2ch Project
 
 ;; Author: Taiki SUGAWARA <taiki@users.sourceforge.net>
 ;; Keywords: network, 2ch
@@ -23,7 +23,7 @@
 ;;; Code:
 (provide 'navi2ch-message)
 (defvar navi2ch-message-ident
-  "$Id: navi2ch-message.el,v 1.30 2002/07/26 13:55:11 nanashi Exp $")
+  "$Id: navi2ch-message.el,v 1.45 2003/09/10 10:14:27 mami Exp $")
 
 (eval-when-compile (require 'cl))
 
@@ -77,6 +77,13 @@
   navi2ch-message-paragraph-separate
   "*`navi2ch-message-mode' $B$G;HMQ$5$l$k(B `paragraph-start'$B!#(B")
 
+(defvar navi2ch-message-sendlog-board
+  `((name . "$BAw?.95$((B")
+    (type . board)
+    (id . "sendlog")
+    (bbstype . localfile)
+    (uri . ,(concat "x-localbbs://" (navi2ch-expand-file-name "sendlog/")))))
+
 (defun navi2ch-message-write-message (board article &optional new sage)
   (when (or (not navi2ch-message-ask-before-write)
 	    (if (functionp navi2ch-message-ask-before-write)
@@ -138,30 +145,46 @@
   (when new
     (insert "Subject: \n"))
   (insert "From: "
-	  (or (cdr (assq 'name navi2ch-message-current-article))
+	  (or (and navi2ch-message-remember-user-name
+		   (cdr (assq 'name navi2ch-message-current-article)))
 	      (cdr (assoc (cdr (assq 'id navi2ch-message-current-board))
 			  navi2ch-message-user-name-alist))
-	      navi2ch-message-user-name "") "\n"
+	      navi2ch-message-user-name "")
+	  "\n"
 	  "Mail: "
 	  (or sage
-	      (cdr (assq 'mail navi2ch-message-current-article))
+	      (and navi2ch-message-remember-user-name
+		   (cdr (assq 'mail navi2ch-message-current-article)))
+	      (cdr (assoc (cdr (assq 'id navi2ch-message-current-board))
+			  navi2ch-message-mail-address-alist))
 	      navi2ch-message-mail-address "")
 	  "\n"
 	  (navi2ch-propertize navi2ch-message-header-separator
 			      'read-only t
-			      'front-sticky t 'rear-nonsticky t)))
+			      'front-sticky t 'rear-nonsticky t))
+  (setq buffer-undo-list nil)
+  (set-buffer-modified-p nil))
+
+(defsubst navi2ch-message-header-end ()
+  (save-restriction
+    (widen)
+    (if (get-text-property (point-min) 'read-only)
+	(point-min)
+      (next-single-property-change (point-min) 'read-only))))
 
 (defun navi2ch-message-cleanup-message ()
   (save-excursion
-    (when navi2ch-message-cleanup-trailing-whitespace
-      (goto-char (point-min))
-      (when (re-search-forward navi2ch-message-header-separator nil t)
+    (let ((start (progn (goto-char (navi2ch-message-header-end))
+			(forward-line)
+			(point))))
+      (when navi2ch-message-cleanup-trailing-whitespace
+	(goto-char start)
 	(while (re-search-forward "[ \t]+$" nil t)
-	  (replace-match ""))))
-    (when navi2ch-message-cleanup-trailing-newline
-      (goto-char (point-min))
-      (if (re-search-forward "[ \t\n]+\\'" nil t)
-	  (replace-match "")))))
+	  (replace-match "")))
+      (when navi2ch-message-cleanup-trailing-newline
+	(goto-char start)
+	(if (re-search-forward "[ \t\n]+\\'" nil t)
+	    (replace-match ""))))))
 
 (defun navi2ch-message-send-message ()
   (interactive)
@@ -171,25 +194,33 @@
 	      (if (functionp navi2ch-message-ask-before-send)
 		  (funcall navi2ch-message-ask-before-send "Send message?")
 		(y-or-n-p "Send message?")))
+      (widen)
       (run-hooks 'navi2ch-message-before-send-hook)
       (navi2ch-message-cleanup-message)
       (save-excursion
-	(let (subject from mail message)
+        (let ((end (navi2ch-message-header-end))
+	      (from "")
+	      (mail "")
+	      subject message)
 	  (goto-char (point-min))
 	  (when navi2ch-message-new-message-p
-	    (re-search-forward "^Subject: ?\\(.*\\)" nil t)
-	    (setq subject (match-string 1)))
-	  (re-search-forward "^From: ?\\(.*\\)")
-	  (setq from (match-string 1))
-	  (when navi2ch-message-remember-user-name
-	    (setq navi2ch-message-user-name from))
-	  (when (not navi2ch-message-new-message-p)
-	    (navi2ch-message-set-name from))
-	  (re-search-forward "^Mail: ?\\(.*\\)")
-	  (setq mail (match-string 1))
-	  (when (not navi2ch-message-new-message-p)
-	    (navi2ch-message-set-mail mail))
-	  (forward-line 2)
+	    (if (re-search-forward "^Subject: ?\\(.*\\)" end t)
+		(setq subject (match-string 1))
+	      (setq subject "")))
+	  (goto-char (point-min))
+	  (when (re-search-forward "^From: ?\\(.*\\)" end t)
+	    (setq from (match-string 1))
+	    (when (and (not navi2ch-message-new-message-p)
+		       navi2ch-message-remember-user-name)
+	      (navi2ch-message-set-name from)))
+	  (goto-char (point-min))
+	  (when (re-search-forward "^Mail: ?\\(.*\\)" end t)
+	    (setq mail (match-string 1))
+	    (when (and (not navi2ch-message-new-message-p)
+		       navi2ch-message-remember-user-name)
+	      (navi2ch-message-set-mail mail)))
+	  (goto-char end)
+	  (forward-line)
 	  (setq message (buffer-substring-no-properties (point) (point-max)))
 	  (let ((buffer (current-buffer))
 		(inhibit-read-only t))
@@ -197,18 +228,20 @@
 	      (set-buffer (get-buffer-create
 			   navi2ch-message-backup-buffer-name))
 	      (erase-buffer)
-	      (insert-buffer buffer)
-	      (bury-buffer)))
+	      (insert-buffer buffer)))
 	  (when navi2ch-message-trip
 	    (setq from (concat from "#" navi2ch-message-trip)))
 	  (let ((board navi2ch-message-current-board)
 		(article navi2ch-message-current-article)
 		result)
-					; $B"-(Bresult$B$r8E$$;EMM$KLa$7$?!#(Bspid$B$O!"(Bnavi2ch-multibbs.el$B$N(B
-					; $B"-(B   navi2ch-2ch-send-message $B$G=hM}$9$k!#(B
+	    ;; $B"-(Bresult$B$r8E$$;EMM$KLa$7$?!#(Bspid$B$O!"(Bnavi2ch-multibbs.el$B$N(B
+	    ;; $B"-(B   navi2ch-2ch-send-message $B$G=hM}$9$k!#(B
 	    (setq result (navi2ch-multibbs-send-message
 			  from mail message subject board article))
 	    (when result
+	      (when navi2ch-message-save-sendlog
+		(navi2ch-message-add-sendlog from mail message subject
+					     board article))
 	      (message "waiting new message...")
 	      (sleep-for navi2ch-message-wait-time)
 	      (message "%s%s" (current-message) "done")
@@ -219,7 +252,9 @@
 		      (navi2ch-board-sync))
 		  (when (buffer-live-p navi2ch-message-current-article-buffer)
 		    (set-buffer navi2ch-message-current-article-buffer)
-		    (navi2ch-article-sync navi2ch-message-force-sync))))))))
+		    (navi2ch-article-sync navi2ch-message-force-sync)))))
+	    (when (get-buffer navi2ch-message-backup-buffer-name)
+	      (bury-buffer navi2ch-message-backup-buffer-name)))))
       (run-hooks 'navi2ch-message-after-send-hook)
       (navi2ch-message-exit 'after-send))))
 
@@ -259,10 +294,12 @@
 (defun navi2ch-message-cite-original-from-number (num &optional arg)
   "$BHV9f$rA*$s$G!"0zMQ$9$k!#(B"
   (interactive "ninput number: \nP")
+  (when (< (point) (navi2ch-message-header-end))
+    (error "Cannot cite in header"))
   (let (same msg board article)
     (save-excursion
       (set-buffer (navi2ch-article-current-buffer))
-      (setq msg (cdr (assq 'data (navi2ch-article-get-message num))))
+      (setq msg (navi2ch-article-get-message-string num))
       (setq article navi2ch-article-current-article)
       (setq board navi2ch-article-current-board)
       (setq same (and (string-equal (cdr (assq 'id board))
@@ -273,7 +310,7 @@
 	(insert ">>" (number-to-string num) "\n")
       (insert (navi2ch-article-to-url board article num num nil) "\n"))
     (unless arg
-      (set-mark (point))
+      (push-mark)
       (let ((point (point)))
 	(insert msg "\n")
 	(string-rectangle point (point) navi2ch-message-cite-prefix)))))
@@ -290,15 +327,15 @@
 	  (set-buffer navi2ch-message-current-article-buffer)
 	(navi2ch-article-view-article navi2ch-message-current-board
 				      navi2ch-message-current-article
-				      navi2ch-message-force-sync))
-      (navi2ch-article-load-number))))
+				      navi2ch-message-force-sync)))
+    (navi2ch-article-load-number)))
 
 (defun navi2ch-message-kill-message (&optional no-ask)
   (when (or no-ask
 	    (not navi2ch-message-ask-before-kill)
 	    (if (functionp navi2ch-message-ask-before-kill)
-		(funcall navi2ch-message-ask-before-kill "Kill current message?"))
-	    (y-or-n-p "Kill current message?"))
+		(funcall navi2ch-message-ask-before-kill "Kill current message?")
+	      (y-or-n-p "Kill current message?")))
     (kill-buffer navi2ch-message-buffer-name)
     t))
 
@@ -357,22 +394,27 @@
 
 (defun navi2ch-message-insert-aa-list ()
   (let ((aa-width navi2ch-message-popup-aa-width)
-	(nl nil))
+	(nl nil)
+	alist keys)
     (dolist (elt (append navi2ch-message-aa-alist
 			 navi2ch-message-aa-default-alist))
-      (let* ((key (car elt))
-	     (val (cdr elt))
-	     string width)
-	(when (and (stringp key) (stringp val))
-	  (setq string (format "%s: %s" (key-description key) val)
-		width (string-width string))
-	  (if (> width aa-width)
-	      (setq string (concat (truncate-string-to-width string
-							     (- aa-width 3))
-				   "...")))
-	  (insert (truncate-string-to-width string aa-width nil ?\ )
-		  (if nl "\n" " "))
-	  (setq nl (not nl)))))))
+      (when (and (not (member (car elt) keys))
+		 (stringp (car elt))
+		 (stringp (cdr elt)))
+	(setq alist (cons elt alist))
+	(setq keys (cons (car elt) keys))))
+    (dolist (key (sort keys 'string<))
+      (let ((val (cdr (assoc key alist)))
+	    string width)
+	(setq string (format "%s: %s" (key-description key) val)
+	      width (string-width string))
+	(if (> width aa-width)
+	    (setq string (concat (truncate-string-to-width string
+							   (- aa-width 3))
+				 "...")))
+	(insert (truncate-string-to-width string aa-width nil ?\ )
+		(if nl "\n" " "))
+	(setq nl (not nl))))))
 
 (defun navi2ch-message-popup-aa-list ()
   "aa $B$N%j%9%H$rI=<($9$k!#(B"
@@ -439,5 +481,63 @@
     (other-window 1)
     (switch-to-buffer (get-buffer navi2ch-message-buffer-name))))
 
+;; sendlog$B5!G=(B
+(defun navi2ch-message-sendlog-subject (board article)
+  ;; $BAw?.$7$?%l%9$rJ]B8$9$k%9%l$N%?%$%H%k$rJV$9!#(B
+  ;; $B%?%$%H%k$r:Y$+$/;XDj$7$?$$$H$-$O$3$N4X?t$r>e=q$-$7$F$M!#(B
+  ;; nil $B$rJV$9$H!"MzNr$OJ]B8$5$l$^$;$s!#(B
+  navi2ch-message-sendlog-subject)
+
+(defun navi2ch-message-sendlog-subject-with-volume (base format limit
+							 subject-list)
+  (let ((subject base)
+	(regexp (concat "\\`"
+			(format (regexp-quote format)
+				(regexp-quote base) "\\([0-9]+\\)")
+			"\\'"))
+	(max 0)
+	article)
+    (when limit
+      (dolist (x subject-list)
+	(let ((sbj (cdr (assq 'subject x)))
+	      num)
+	  (when (and (or (and (string= base sbj)
+			      (setq num 1))
+			 (and (string-match regexp sbj)
+			      (setq num
+				    (string-to-number (match-string 1 sbj)))))
+		     (> num max))
+	    (setq max num
+		  article x))))
+      (when article
+	(if (>= (string-to-number (cdr (assq 'response article))) limit)
+	    (setq subject (format format base (number-to-string (1+ max))))
+	  (setq subject (cdr (assq 'subject article))))))
+    subject))
+
+(defun navi2ch-message-add-sendlog (from mail message subject board article)
+  (let ((navi2ch-localfile-default-file-modes ?\700)
+	;; $BAw?.95$((B $B$N%9%l%?%$$K(B &hearts; $B$H$+$r;H$($k$h$&$K!#(B
+	(navi2ch-decode-character-references nil)
+	(url (navi2ch-article-to-url board article))
+	(sbj (or subject (cdr (assq 'subject article))))
+	(lsubject (navi2ch-message-sendlog-subject board article))
+	(lboard navi2ch-message-sendlog-board)
+	(fmt navi2ch-message-sendlog-volume-format)
+	(limit navi2ch-message-sendlog-response-limit)
+	larticle lsbj-list)
+    (when (and lsubject lboard)
+      (setq message (format "Subject: %s\nURL: %s\n\n%s" sbj url message)
+	    lsbj-list (navi2ch-board-get-updated-subject-list lboard)
+	    lsubject (navi2ch-message-sendlog-subject-with-volume
+		      lsubject fmt limit lsbj-list))
+      (catch 'loop
+	(dolist (s lsbj-list)
+	  (when (string= (cdr (assq 'subject s)) lsubject)
+	    (throw 'loop (setq larticle s)))))
+      (when larticle (setq lsubject nil))
+      (navi2ch-multibbs-send-message from mail message
+				     lsubject lboard larticle))))
+
 (run-hooks 'navi2ch-message-load-hook)
 ;;; navi2ch-message.el ends here
