@@ -1,6 +1,6 @@
 ;;; navi2ch-mona.el --- Mona Font Utils for Navi2ch
 
-;; Copyright (C) 2001, 2002 by Navi2ch Project
+;; Copyright (C) 2001-2003 by Navi2ch Project
 ;; Copyright (C) 1995,1996,1997,1998,1999 Free Software Foundation, Inc.
 
 ;; Author: Taiki SUGAWARA <taiki@users.sourceforge.net>
@@ -40,7 +40,7 @@
 ;;; Code:
 (provide 'navi2ch-mona)
 (defvar navi2ch-mona-ident
-  "$Id: navi2ch-mona.el,v 1.26 2002/11/11 09:48:17 tabmore Exp $")
+  "$Id: navi2ch-mona.el,v 1.30 2003/07/16 12:30:48 mami Exp $")
 (eval-when-compile (require 'cl))
 (require 'base64)
 (require 'navi2ch-util)
@@ -143,6 +143,21 @@ nil $B$N$H$-$O(B `navi2ch-mona-disable-board-list' $B$G;XDj$7$?HD0J30$N(B
   :type '(repeat (string :tag "$BHD(B"))
   :group 'navi2ch-mona)
 
+(defcustom navi2ch-mona-enable-article-list nil
+  "*$B%b%J!<%U%)%s%H$GI=<($9$k%9%l$N%j%9%H!#(B
+`navi2ch-mona-disable-board-list' $B$h$j$bM%@h$5$l$k!#(B"
+  :type '(repeat (cons (string :tag "$BHD(B")
+		       (string :tag "$B%9%l(B")))
+  :group 'navi2ch-mona)
+
+(defcustom navi2ch-mona-disable-article-list nil
+  "*$B%b%J!<%U%)%s%H$r;H$o$J$$%9%l$N%j%9%H!#(B
+`navi2ch-mona-enable-board-list', `navi2ch-mona-enable-article-list'
+$B$h$j$bM%@h$5$l$k!#(B"
+  :type '(repeat (cons (string :tag "$BHD(B")
+		       (string :tag "$B%9%l(B")))
+  :group 'navi2ch-mona)
+
 (defcustom navi2ch-mona-pack-space-p nil
   "*non-nil $B$J$i!"(BWeb $B%V%i%&%6$N$h$&$K(B2$B$D0J>e$N6uGr$O(B1$B$D$K$^$H$a$FI=<($9$k!#(B"
   :type 'boolean
@@ -288,10 +303,19 @@ nil is returned.  Otherwise the associated face object is returned."
 
 (defun navi2ch-mona-arrange-message ()
   "$B%b%J!<%U%)%s%H$r;H$&HD$J$i$=$N$?$a$N4X?t$r8F$V!#(B"
-  (let ((id (cdr (assq 'id navi2ch-article-current-board))))
-    (when (and (or (not navi2ch-mona-enable-board-list)
-		   (member id navi2ch-mona-enable-board-list))
-	       (not (member id navi2ch-mona-disable-board-list)))
+  (let ((id (cdr (assq 'id (if (eq major-mode 'navi2ch-popup-article-mode)
+			       navi2ch-popup-article-current-board
+			     navi2ch-article-current-board))))
+	(artid (cdr (assq 'artid (if (eq major-mode 'navi2ch-popup-article-mode)
+				     navi2ch-popup-article-current-article
+				   navi2ch-article-current-article)))))
+    (when (and (or (and (or (not navi2ch-mona-enable-board-list)
+			    (member id navi2ch-mona-enable-board-list))
+			(not (member id navi2ch-mona-disable-board-list)))
+		   (member (cons id artid)
+			   navi2ch-mona-enable-article-list))
+	       (not (member (cons id artid)
+			    navi2ch-mona-disable-article-list)))
       (navi2ch-mona-put-face))
     (when navi2ch-mona-pack-space-p
       (navi2ch-mona-pack-space))))
