@@ -1,6 +1,6 @@
 ;;; navi2ch-net.el --- Network module for navi2ch
 
-;; Copyright (C) 2000-2002 by Navi2ch Project
+;; Copyright (C) 2000-2003 by Navi2ch Project
 
 ;; Author: Taiki SUGAWARA <taiki@users.sourceforge.net>
 ;; Keywords: network 2ch
@@ -23,7 +23,7 @@
 ;;; Code:
 (provide 'navi2ch-net)
 (defvar navi2ch-net-ident
-  "$Id: navi2ch-net.el,v 1.66 2002/12/15 04:23:00 nanashi Exp $")
+  "$Id: navi2ch-net.el,v 1.80 2003/09/06 14:58:17 mami Exp $")
 
 (eval-when-compile (require 'cl))
 (require 'timezone)
@@ -34,6 +34,7 @@
 (defvar navi2ch-net-connection-name "navi2ch connection")
 (defvar navi2ch-net-user-agent "Monazilla/1.00 Navi2ch")
 (defvar navi2ch-net-setting-file-name "SETTING.TXT")
+(defvar navi2ch-net-last-date nil)
 (defvar navi2ch-net-last-url nil)
 (defvar navi2ch-net-process nil)
 (defvar navi2ch-net-last-host nil)
@@ -187,11 +188,12 @@ BODY $B$NI>2ACf$K%(%i!<$,5/$3$k$H(B nil $B$rJV$9!#(B"
            (cons 'file url)
            (cons 'port (string-to-number (match-string 3 proxy)))
            (cons 'host2ch host2ch)))
-      (string-match "http://\\([^/]+\\)\\(.*\\)" url)
+      (string-match "http://\\([^/:]+\\):?\\([0-9]+\\)?\\(.*\\)" url)
       (list
        (cons 'host (match-string 1 url))
-       (cons 'file (match-string 2 url))
-       (cons 'port  80)
+       (cons 'port (string-to-number (or (match-string 2 url)
+					 "80")))
+       (cons 'file (match-string 3 url))
        (cons 'host2ch host2ch)))))
 
 (defun navi2ch-net-http-proxy-basic-credentials (user pass)
@@ -250,6 +252,9 @@ BODY $B$NI>2ACf$K%(%i!<$,5/$3$k$H(B nil $B$rJV$9!#(B"
 	     (while (re-search-forward "^\\([^\r\n:]+\\): \\(.+\\)\r\n" end t)
 	       (setq list (cons (cons (match-string 1) (match-string 2))
 				list)))
+	     (let ((date (assoc-ignore-case "Date" list)))
+	       (when (and date (stringp (cdr date)))
+		 (setq navi2ch-net-last-date (cdr date))))
 	     (setq navi2ch-net-header (nreverse list))))))))
 
 (defun navi2ch-net-get-content-subr-with-temp-file (gzip-p start end)
@@ -288,8 +293,8 @@ chunk $B$N%5%$%:$rJV$9!#(Bpoint $B$O(B chunk $B$ND>8e$K0\F0!#(B"
   (catch 'ret
     (let ((p (point))
 	  size end)
-      (while (and (eq (process-status proc) 'open)
-		  (not (looking-at "\\([0-9a-fA-F]+\\)[^\r\n]*\r\n")))
+      (while (and (not (looking-at "\\([0-9a-fA-F]+\\)[^\r\n]*\r\n"))
+		  (eq (process-status proc) 'open))
 	(accept-process-output proc)
 	(goto-char p))
       (when (not (match-string 1))
@@ -348,6 +353,7 @@ chunk $B$N%5%$%:$rJV$9!#(Bpoint $B$O(B chunk $B$ND>8e$K0\F0!#(B"
 		      (goto-char (+ p size))))
 		   ((or (string= (navi2ch-net-get-protocol proc)
 				 "HTTP/1.0")
+			(not navi2ch-net-enable-http11)
 			(and (stringp (cdr (assoc "Connection" header)))
 			     (string= (cdr (assoc "Connection" header))
 				      "close")))
@@ -378,6 +384,9 @@ OTHER-HEADER $B$,(B `non-nil' $B$J$i$P%j%/%(%9%H$K$3$N%X%C%@$rDI2C$9$k!#(B
 			 (cons "Pragma" "no-cache")
 		       (and time (cons "If-Modified-Since" time)))
 		     (and navi2ch-net-accept-gzip
+			  ;; regexp $B$OJQ?t$K$7$?J}$,$$$$$N$+$J!#$$$$JQ?tL>$,;W$$$D$+$J$$!#(B
+			  (not (string-match "\\.gz$" url))
+			  (not (assoc "Range" other-header))
 			  '("Accept-Encoding" . "gzip"))
 		     (and navi2ch-net-user-agent
 			  (cons "User-Agent" navi2ch-net-user-agent)))
@@ -502,9 +511,11 @@ header $B$KD9$5$,4^$^$l$F$$$J$$>l9g$O(B nil $B$rJV$9!#(B"
 	(>= len (or size 0))
       t)))				; $B%[%s%H$K$3$l$G$$$$$+$J(B?
 
-(defun navi2ch-net-update-file-diff (url file &optional time)
+(defun navi2ch-net-update-file-diff (url file &optional time func)
   "FILE $B$r:9J,$G99?7$9$k!#(B
 TIME $B$,(B `non-nil' $B$J$i$P(B TIME $B$h$j?7$7$$;~$@$199?7$9$k!#(B
+FUNC $B$,(B non-nil $B$J$i$P99?78e(B FUNC $B$r;H$C$F%U%!%$%k$rJQ49$9$k!#(B
+FUNC $B$O(B current-buffer $B$rA`:n$9$k4X?t$G$"$k;v!#(B
 $B99?7$G$-$l$P(B HEADER $B$rJV$9!#(B"
   (let ((dir (file-name-directory file)))
     (unless (file-exists-p dir)
@@ -543,14 +554,29 @@ TIME $B$,(B `non-nil' $B$J$i$P(B TIME $B$h$j?7$7$$;~$@$199?7$9$k!#(B
 			(with-temp-file file
 			  (insert-file-contents file nil nil size)
 			  (goto-char (point-max))
+			  (when (and cont func)
+			    (message "%stranslating..." (current-message))
+			    (setq cont (with-temp-buffer
+					 (insert cont)
+					 (goto-char (point-min))
+					 (funcall func)
+					 (buffer-string))))
 			  (insert cont))
 			(message "%sdone" (current-message)))))))
 	  ((string= status "200")
 	   (if (not (navi2ch-net-check-aborn size header))
 	       (setq aborn-p t)
 	     (message "%s: getting whole file..." (current-message))
-	     (with-temp-file file
-	       (insert (navi2ch-net-get-content proc)))
+	     (let ((cont (navi2ch-net-get-content proc)))
+	       (when (and cont func)
+		 (message "%stranslating..." (current-message))
+		 (setq cont (with-temp-buffer
+			      (insert cont)
+			      (goto-char (point-min))
+			      (funcall func)
+			      (buffer-string))))
+	       (with-temp-file file
+		 (insert cont)))
 	     (message "%sdone" (current-message))))
 	  ((string= status "304")
 	   (setq header (navi2ch-net-add-state 'not-updated header)))
@@ -559,14 +585,17 @@ TIME $B$,(B `non-nil' $B$J$i$P(B TIME $B$h$j?7$7$$;~$@$199?7$9$k!#(B
     (if (not aborn-p)
 	header
       (message "$B$"$\!<$s(B!!!")
-      (when (and navi2ch-net-save-old-file-when-aborn
-		 (or (not (eq navi2ch-net-save-old-file-when-aborn
-			      'ask))
-		     (y-or-n-p "$B$"$\!<$s(B!!! backup old file? ")))
-	(copy-file file (read-file-name "file name: ")))
+      (navi2ch-net-save-aborn-file file)
       (navi2ch-net-add-state
        'aborn
-       (navi2ch-net-update-file url file nil nil)))))
+       (navi2ch-net-update-file url file nil func)))))
+
+(defun navi2ch-net-save-aborn-file (file)
+  (when (and navi2ch-net-save-old-file-when-aborn
+	     (or (not (eq navi2ch-net-save-old-file-when-aborn
+			  'ask))
+		 (y-or-n-p "$B$"$\!<$s(B!!! backup old file? ")))
+    (copy-file file (read-file-name "file name: "))))
 
 (defun navi2ch-net-update-file-with-readcgi (url file &optional time diff)
   "FILE $B$r(B URL $B$+$i(B read.cgi $B$r;H$C$F99?7$9$k!#(B
@@ -693,8 +722,13 @@ internet drafts directory for a copy.")
 				   navi2ch-coding-system)))
     (cond ((string-match "$B#E#R#R#O#R!'(B\\([^<]+\\)" str)
 	   (match-string 1 str))
+	  ;; Samba24 http://age.s22.xrea.com/talk2ch/new.txt
+	  ((string-match "$B#E#R#R#O#R(B - \\([^<\n]+\\)" str)
+	   (match-string 1 str))
 	  ((string-match "<b>\\([^<]+\\)" str)
-	   (match-string 1 str)))))
+	   (match-string 1 str))
+	  ((string-match "\\([^<>\n]+\\)<br>\\([^<>]+\\)<hr>"  str)
+	   (concat (match-string 1 str) (match-string 2 str))))))
 
 ;; Set-Cookie: SPID=6w9HFhEM; expires=Tuesday, 23-Apr-2002 00:00:00 GMT; path=/
 (defun navi2ch-net-send-message-get-spid (proc)
@@ -706,56 +740,6 @@ internet drafts directory for a copy.")
 		((string-match "^PON=\\([^;]+\\);" str)
 		 (return (match-string 1 str))))))))
 
-(defun navi2ch-net-send-message (from mail message subject url referer bbs key spid)
-  "$B%a%C%;!<%8$rAw$k!#(B
-$BAw?.@.8y$J$i(B t $B$rJV$9(B"
-  (navi2ch-net-ignore-errors
-   (let ((param-alist
-	  (list
-	   (cons "submit" "$B=q$-9~$`(B")
-	   (cons "FROM" (or from ""))
-	   (cons "mail" (or mail ""))
-	   (cons "bbs" bbs)
-	   (cons "time"
-		 (mapconcat 'int-to-string
-			    (let ((time (current-time)))
-			      (navi2ch-bigint-add
-			       (navi2ch-bigint-multiply
-				(nth 0 time) (expt 2 16)) (nth 1 time)))
-			    ""))
-	   (cons "MESSAGE" message)
-	   (if subject
-	       (cons "subject" subject)
-	     (cons "key" key))))
-	 (navi2ch-net-http-proxy (if navi2ch-net-send-message-use-http-proxy
-				     navi2ch-net-http-proxy)))
-     (unwind-protect
-	 (let ((proc (navi2ch-net-send-request
-		      url "POST"
-		      (list (cons "Content-Type"
-				  "application/x-www-form-urlencoded")
-			    (cons "Cookie"
-				  (concat "NAME=" from
-					  "; MAIL=" mail
-					  (if spid
-					      (concat "; SPID=" spid))))
-			    (cons "Referer" referer))
-		      (navi2ch-net-get-param-string param-alist))))
-	   (message "send message...")
-	   (setq spid (navi2ch-net-send-message-get-spid proc))
-	   (cons
-	    (if (navi2ch-net-send-message-success-p proc)
-		(progn
-		  (message "send message...succeed")
-		  t)
-	      (let ((err (navi2ch-net-send-message-error-string proc)))
-		(if (stringp err)
-		    (message "send message...failed: %s" err)
-		  (message "send message...failed")))
-	      nil)
-	    spid))
-       (navi2ch-net-cleanup-process))))) ; $BG0$N$?$a@\B3$r@Z$k!#(B
-
 (defun navi2ch-net-download-logo (board)
   (let* ((coding-system-for-read 'binary)
 	 (coding-system-for-write 'binary)
@@ -766,17 +750,17 @@ internet drafts directory for a copy.")
 	 src)
     (when (string-match
 	   "BBS_\\(TITLE_PICTURE\\|FIGUREHEAD\\)=\\(.+\\)" content)
-      (setq src (match-string 2 content)))
-    (let (url file)
-      (setq url (if (string-match "http://" src)
-		    src
-		  (navi2ch-board-get-url board src)))
-      (string-match "/\\([^/]+\\)$" url)
-      (setq file (match-string 1 url))
-      (when file
-	(setq file (navi2ch-board-get-file-name board file))
-	(when (navi2ch-net-update-file url file nil nil t)
-	  file)))))
+      (setq src (match-string 2 content))
+      (let (url file)
+	(setq url (if (string-match "http://" src)
+		      src
+		    (navi2ch-board-get-url board src)))
+	(string-match "/\\([^/]+\\)$" url)
+	(setq file (match-string 1 url))
+	(when file
+	  (setq file (navi2ch-board-get-file-name board file))
+	  (when (navi2ch-net-update-file url file nil nil t)
+	    file))))))
 
 (defun navi2ch-net-add-state (state header)
   "HEADER $B$K(B STATE $B$rDI2C$9$k!#(B"
