@@ -1,6 +1,6 @@
 ;;; navi2ch-vars.el --- User variables for navi2ch.
 
-;; Copyright (C) 2001, 2002 by Navi2ch Project
+;; Copyright (C) 2001-2003 by Navi2ch Project
 
 ;; Author: Taiki SUGAWARA <taiki@users.sourceforge.net>
 ;; Keywords: www 2ch
@@ -27,7 +27,7 @@
 ;;; Code:
 (provide 'navi2ch-vars)
 (defvar navi2ch-vars-ident
-  "$Id: navi2ch-vars.el,v 1.99 2002/11/16 18:01:04 mami Exp $")
+  "$Id: navi2ch-vars.el,v 1.127 2003/09/10 10:14:27 mami Exp $")
 
 (defconst navi2ch-on-xemacs (featurep 'xemacs))
 (defconst navi2ch-on-emacs21 (and (not navi2ch-on-xemacs)
@@ -59,12 +59,12 @@
   :group 'navi2ch)
 
 (defgroup navi2ch-message nil
-  "*Navi2ch, article buffer."
+  "*Navi2ch, message buffer."
   :prefix "navi2ch-"
   :group 'navi2ch)
 
 (defgroup navi2ch-net nil
-  "*Navi2ch, article buffer."
+  "*Navi2ch, networking."
   :prefix "navi2ch-"
   :group 'navi2ch)
 
@@ -95,6 +95,16 @@
   :type '(repeat :tag "$B0z?t(B" string)
   :group 'navi2ch)
 
+(defcustom navi2ch-bzip2-program "bzip2"
+  "*bzip2 $B$K;H$&%W%m%0%i%`!#(B"
+  :type 'string
+  :group 'navi2ch)
+
+(defcustom navi2ch-bzip2-args '("-d" "-c" "-q")
+  "*bzip2 $B$r8F$S=P$9$H$-$N0z?t!#(B"
+  :type '(repeat :tag "$B0z?t(B" string)
+  :group 'navi2ch)
+
 (defcustom navi2ch-init-file (concat
                               (file-name-as-directory navi2ch-directory)
                               "init")
@@ -168,6 +178,20 @@ nil $B$N>l9g$O(B browse-url-browser-function $B$r;H$&!#(B
   :type 'string
   :group 'navi2ch)
 
+(defcustom navi2ch-file-name-reserved-char-regexp ":"
+  "*navi2ch-expand-file-name $B$G%(%9%1!<%W$9$kJ8;z$rI=$9@55,I=8=!#(B
+$B%G%U%)%k%H$O(B \":\" $B$G!"(BURL $BCf$N(B \":\" $B$,%G%#%l%/%H%jL>$G$O(B \"%3A\" $B$K%(%9%1!<%W$5$l$k!#(B
+\"~\" $B$b%(%9%1!<%W$7$?$$$H$-$O(B \"[:~]\" $B$r;XDj$9$k!#(B"
+  :type 'regexp
+  :group 'navi2ch)
+
+(defcustom navi2ch-decode-character-references t
+  "*non-nil$B$J$i!"?tCMJ8;z;2>H!"J8;z<BBN;2>H$NI=<($r;n$_$k!#(B
+GNU Emacs 21, XEmacs 21.5 $B0J9_$G$"$l$P%G%U%)%k%H$GI=<($G$-$^$9$,!"(B
+$B$=$l0JA0$N(BEmacsen$B$G$O(BMule-UCS$B$,I,MW$G$9!#(B (require 'un-define) $B$7$F$M!#(B"
+  :type 'boolean
+  :group 'navi2ch)
+
 ;;; list variables
 (defcustom navi2ch-list-window-width 20
   "*$BHD0lMw%&%#%s%I%&$N2#I}!#(B"
@@ -222,7 +246,7 @@ nil $B$J$i$P<jF0$G99?7$7$J$$$+$.$j<h$j$K$$$+$J$$!#(B"
 
 (defcustom navi2ch-list-valid-host-regexp
   (concat "\\("
-	  (regexp-opt '(".2ch.net" ".bbspink.com" ".machibbs.com"))
+	  (regexp-opt '(".2ch.net" ".bbspink.com" ".machibbs.com" ".machi.to"))
 	  "\\)\\'")
   "*$B#2$A$c$s$M$k$NHD$H$_$J$9%[%9%H$N@55,I=8=!#(B"
   :type 'regexp
@@ -239,6 +263,21 @@ nil $B$J$i$P<jF0$G99?7$7$J$$$+$.$j<h$j$K$$$+$J$$!#(B"
 		 (const :tag "$B%U%'%$%9$r;HMQ$7$J$$(B" nil))
   :group 'navi2ch-list)
 
+(defcustom navi2ch-list-filter-list nil
+  "*$B%9%l%C%I$NJ,N`0lMw$r$$$8$k%U%#%k%?!<$N%j%9%H!#(B
+$B$=$l$>$l$N%U%#%k%?!<$O(B elisp $B$N4X?t$J$i$P(B $B$=$N(B symbol$B!"(B
+$B30It%W%m%0%i%`$r8F$V$J$i(B
+'(\"perl\" \"2ch.pl\")
+$B$H$$$C$?46$8$N(B list $B$r@_Dj$9$k!#(B
+$BNc$($P$3$s$J46$8!#(B
+\(setq navi2ch-list-filter-list
+      '(navi2ch-filter
+        (\"perl\" \"2ch.pl\")
+        (\"perl\" \"filter-with-board.pl\" \"-b\" board)
+        ))"
+  :type '(repeat sexp)
+  :group 'navi2ch-list)
+
 ;;; board variables
 (defcustom navi2ch-board-max-line nil
   "*$B%@%&%s%m!<%I$9$k(B subject.txt $B$N9T?t!#(Bnil $B$J$iA4It%@%&%s%m!<%I$9$k!#(B"
@@ -312,7 +351,7 @@ non-nil $B$J$i2<$K0\F0$9$k(B
   :group 'navi2ch-board)
 
 (defcustom navi2ch-history-max-line 100
-  "*$B%R%9%H%j$N9T?t$N@)8B8B!#(Bnil $B$J$i$P@)8B$7$J$$!#(B"
+  "*$B%R%9%H%j$N9T?t$N@)8B!#(Bnil $B$J$i$P@)8B$7$J$$!#(B"
   :type '(choice (integer :tag "$B:GBg$N9T?t$r;XDj(B")
 		 (const :tag "$B@)8B$7$J$$(B" nil))
   :group 'navi2ch-board)
@@ -350,13 +389,13 @@ non-nil $B$J$i$P5-21$9$k!#(B"
   :group 'navi2ch-board)
 
 (defcustom navi2ch-board-expire-bookmark-p nil
-  "*expire $B$9$k$H$-$K(B bookarmk $B$5$l$F$$$k%9%l$b(B expire $B$9$k$+$I$&$+!#(B
+  "*expire $B$9$k$H$-$K(B bookmark $B$5$l$F$$$k%9%l$b(B expire $B$9$k$+$I$&$+!#(B
 non-nil $B$J$i$P(B expire $B$9$k!#(B"
   :type 'boolean
   :group 'navi2ch-board)
 
 (defcustom navi2ch-board-name-from-file "From File"
-  "*$B%U%!%$%k$+$iFI$_9~$s$@%9%l$rI=$o$9HDL>!#(B"
+  "*$B%U%!%$%k$+$iFI$_9~$s$@%9%l$rI=$9HDL>!#(B"
   :type 'string
   :group 'navi2ch-board)
 
@@ -367,20 +406,28 @@ non-nil $B$J$i$P(B expire $B$9$k!#(B"
   :group 'navi2ch-board)
 
 (defcustom navi2ch-bm-sort-by-state-order
-  '(("+U" . 0)
-    ("+V" . 1)
-    ("+C" . 2)
-    ("+ " . 3)
-    (" U" . 4)
-    (" V" . 5)
-    (" C" . 6)
-    ("  " . 7)
-    ("=U" . 8)
-    ("=V" . 9)
-    ("=C" . 10)
-    ("= " . 11))
+  '(("%U" . 0)
+    ("+U" . 1)
+    ("%V" . 2)
+    ("+V" . 3)
+    ("%C" . 4)
+    ("+C" . 5)
+    ("% " . 6)
+    ("+ " . 7)
+    (" U" . 8)
+    (" V" . 9)
+    (" C" . 10)
+    ("  " . 11)
+    ("=U" . 12)
+    ("=V" . 13)
+    ("=C" . 14)
+    ("= " . 15))
   "*$B>uBV$G%=!<%H$9$k$H$-$N=g=x$r7h$a$k%j%9%H!#(B"
-  :type '(list (cons (const :tag "$B>uBV(B +U" "+U") (number :tag "$B=gHV(B"))
+  :type '(list (cons (const :tag "$B>uBV(B %U" "%U") (number :tag "$B=gHV(B"))
+	       (cons (const :tag "$B>uBV(B %V" "%V") (number :tag "$B=gHV(B"))
+	       (cons (const :tag "$B>uBV(B %C" "%C") (number :tag "$B=gHV(B"))
+	       (cons (const :tag "$B>uBV(B % " "% ") (number :tag "$B=gHV(B"))
+	       (cons (const :tag "$B>uBV(B +U" "+U") (number :tag "$B=gHV(B"))
 	       (cons (const :tag "$B>uBV(B +V" "+V") (number :tag "$B=gHV(B"))
 	       (cons (const :tag "$B>uBV(B +C" "+C") (number :tag "$B=gHV(B"))
 	       (cons (const :tag "$B>uBV(B + " "+ ") (number :tag "$B=gHV(B"))
@@ -395,6 +442,43 @@ non-nil $B$J$i$P(B expire $B$9$k!#(B"
 	       (cons (const :tag "$B>uBV(B   " "  ") (number :tag "$B=gHV(B")))
   :group 'navi2ch-board)
 
+(defcustom navi2ch-board-filter-list nil
+  "*$B%9%l%C%I$N0lMw$r$$$8$k%U%#%k%?!<$N%j%9%H!#(B
+$B$=$l$>$l$N%U%#%k%?!<$O(B elisp $B$N4X?t$J$i$P(B $B$=$N(B symbol$B!"(B
+$B30It%W%m%0%i%`$r8F$V$J$i(B
+'(\"perl\" \"2ch.pl\")
+$B$H$$$C$?46$8$N(B list $B$r@_Dj$9$k!#(B
+$BNc$($P$3$s$J46$8!#(B
+\(setq navi2ch-board-filter-list
+      '(navi2ch-filter
+        (\"perl\" \"2ch.pl\")
+        (\"perl\" \"filter-with-board.pl\" \"-b\" board)
+        ))"
+  :type '(repeat sexp)
+  :group 'navi2ch-board)
+
+(defcustom navi2ch-board-check-article-update-suppression-length nil
+  "*$B%9%l$r99?7$9$k:]!"%U%#%k%?!<=hM}$r%A%'%C%/$9$k?7Ce%l%9?t!#(B
+
+$B$?$H$($P(B 10 $B$r;XDj$9$k$H!"(B
+$B%9%l$N?7Ce%l%9$,(B10$B8D0J2<$G$=$N$9$Y$F$,HsI=<($K$J$k$H$-$O!"(B
+$B?7Ce$J$7$H8+$J$5$l$k!#(B
+
+nil $B$r;XDj$9$k$H!"?7Ce%l%9$X$N%U%#%k%?!<=hM}$r%A%'%C%/$7$J$$!#(B"
+  :type '(choice (integer :tag "$B?7Ce%l%9?t(B")
+		 (const :tag "$B%A%'%C%/$7$J$$(B" nil))
+  :group 'navi2ch-board)
+
+(defcustom navi2ch-board-insert-subject-with-diff nil
+  "*non-nil $B$J$i!"(BBoard $B%b!<%I$N%l%9?tMs$K%l%9$NA}2C?t$rI=<($9$k!#(B"
+  :type 'boolean
+  :group 'navi2ch-board)
+
+(defcustom navi2ch-board-insert-subject-with-unread nil
+  "*non-nil $B$J$i!"(BBoard $B%b!<%I$N%l%9?tMs$K%l%9$NL$FI?t$rI=<($9$k!#(B"
+  :type 'boolean
+  :group 'navi2ch-board)
+
 ;;; article variables
 (defcustom navi2ch-article-aadisplay-program
   (if (eq window-system 'w32)
@@ -498,7 +582,7 @@ ask $B$J$iL@<(E*$K0\F0$9$k;~0J30$J$i<ALd$9$k(B
   :group 'navi2ch-article)
 
 (defcustom navi2ch-article-auto-range t
-  "*non-nil $B$J$i!"$^$?I=<($7$F$J$$%9%l%C%I$NI=<(HO0O$r>!<j$K69$a$k!#(B"
+  "*non-nil $B$J$i!"$^$@I=<($7$F$J$$%9%l%C%I$NI=<(HO0O$r>!<j$K69$a$k!#(B"
   :type 'boolean
   :group 'navi2ch-article)
 
@@ -523,24 +607,34 @@ ask $B$J$iL@<(E*$K0\F0$9$k;~0J30$J$i<ALd$9$k(B
   :type 'regexp
   :group 'navi2ch-article)
 
-(defcustom navi2ch-article-number-prefix-regexp "[>$B!d(B<$B!c(B][>$B!d(B<$B!c(B]* *"
-  "*$BF1$8%9%lFb$X$N%j%s%/$rI=$o$9@55,I=8=!#(B"
+(defcustom navi2ch-article-number-prefix-regexp "[>$B!d"d(B<$B!c(B][>$B!d"d(B<$B!c(B]* *"
+  "*$BF1$8%9%lFb$X$N%j%s%/$rI=$9@55,I=8=!#(B"
   :type 'regexp
   :group 'navi2ch-article)
 
-(defcustom navi2ch-article-number-separator-regexp " *, *"
-  "*$BF1$8%9%lFb$X$N%j%s%/$N?t;z$r6h@Z$kJ8;zNs$rI=$o$9@55,I=8=!#(B"
+(defcustom navi2ch-article-number-separator-regexp " *[,$B!"(B=$B!a(B] *"
+  "*$BF1$8%9%lFb$X$N%j%s%/$N?t;z$r6h@Z$kJ8;zNs$rI=$9@55,I=8=!#(B"
   :type 'regexp
   :group 'navi2ch-article)
 
 (defcustom navi2ch-article-number-number-regexp
   "\\([0-9$B#0(B-$B#9(B]+\\(-[0-9$B#0(B-$B#9(B]+\\)?\\)"
-  "*$BF1$8%9%lFb$X$N%j%s%/$N?t;z$rI=$o$9@55,I=8=!#(B"
+  "*$BF1$8%9%lFb$X$N%j%s%/$N?t;z$rI=$9@55,I=8=!#(B"
   :type 'regexp
   :group 'navi2ch-article)
 
+(defcustom navi2ch-article-select-current-link-number-style 'auto
+  "*$B%9%lFb%j%s%/(B (>>3 $B$H$+(B) $B$r$?$I$k$H$-$NI=<(J}K!!#(B
+'popup $B$J$i$D$M$KJL%&%#%s%I%&$r(B popup $B$9$k!#(B
+'jump $B$J$i(B popup $B$;$:$K0\F0$9$k!#(B
+'auto $B$J$i<+F0$G@Z$jBX$($k!#(B"
+  :type '(choice (const :tag "Popup" popup)
+                 (const :tag "Jump" jump)
+		 (const :tag "Auto" auto))
+  :group 'navi2ch-article)
+
 (defcustom navi2ch-article-url-regexp
-  "\\(h?ttps?\\|x-localbbs\\)\\(://[-a-zA-Z0-9_=?#$@~`%&*+|\\/.,:]+\\)"
+  "\\(h?ttps?\\|x-localbbs\\)\\(://[-a-zA-Z0-9_=?#$@~`%&*+|\\/.,:;]+\\)"
   "*$B%l%9$N%F%-%9%H$N$&$A(B URL $B$H$_$J$9ItJ,$N@55,I=8=!#(B"
   :type 'regexp
   :group 'navi2ch-article)
@@ -630,8 +724,14 @@ window $B$NI}$$$C$Q$$$K$7$?$$$J$i(B
                  (sexp :tag "$B4X?t$H$+(B"))
   :group 'navi2ch-article)
 
-(defcustom navi2ch-article-auto-decode-base64-p nil
-  "*non-nil $B$J$i!"%9%l$N(B BASE64 $B$G%(%s%3!<%I$5$l$?%F%-%9%H$r<+F0E83+$9$k!#(B"
+(defcustom navi2ch-article-auto-decode-p nil
+  "*non-nil $B$J$i!"%9%l$N%(%s%3!<%I$5$l$?%F%-%9%H$r<+F0E83+$9$k!#(B"
+  :type 'boolean
+  :group 'navi2ch-article)
+
+(defcustom navi2ch-article-auto-decode-insert-text nil
+  "*non-nil $B$J$i!"<+F0E83+$7$?%F%-%9%H$r%P%C%U%!$KA^F~$9$k!#(B
+`navi2ch-article-auto-decode-p' $B$,(B non-nil $B$N$H$-$N$_8z2L$,$"$k!#(B"
   :type 'boolean
   :group 'navi2ch-article)
 
@@ -657,73 +757,490 @@ nil $B$N>l9g$OF1$8%9%l$NFbMF$N$_$rF@$k!#(B"
   :type 'boolean
   :group 'navi2ch-article)
 
-(defcustom navi2ch-article-decode-character-references t
-  "*non-nil$B$J$i!"?tCMJ8;z;2>H!"J8;z<BBN;2>H$NI=<($r;n$_$k!#(B
-GNU Emacs 21, XEmacs 21.5 $B0J9_$G$"$l$P%G%U%)%k%H$GI=<($G$-$^$9$,!"(B
-$B$=$l0JA0$N(BEmacsen$B$G$O(BMule-UCS$B$,I,MW$G$9!#(B (require 'un-define) $B$7$F$M!#(B"
-  :type 'boolean
-  :group 'navi2ch-article)
-
 (defcustom navi2ch-article-message-filter-list
   '(navi2ch-article-message-filter-by-name
+    navi2ch-article-message-filter-by-mail
     navi2ch-article-message-filter-by-id
-    navi2ch-article-message-filter-by-message)
+    navi2ch-article-message-filter-by-message
+    navi2ch-article-message-filter-by-subject)
   "*$B%l%9$r%U%#%k%?$9$k$?$a$N4X?t$N%j%9%H!#(B
 $B%j%9%H$N(B member $B$H$J$k4X?t$H$7$F$O!"(B
 $B%l%9$N(B alist $B$r0z$-?t$K<h$j!"(B
 $B%U%#%k%?8e$NJ8;zNs$b$7$/$O(B 'hide $B$+(B 'important $B$rJV$9$b$N$r;XDj$9$k!#(B
 
 $B2<5-$OL>A0Ms$,!V$[$2!W$N$H$-$K!V$"$\$\!<$s!W$KCV49$9$k$?$a$N(B
-($B$A$g$C$H>iD9$J(B) $B4X?t$NNc!#(B
+\($B$A$g$C$H>iD9$J(B) $B4X?t$NNc!#(B
 
-(defun my-navi2ch-article-message-filter-hoge (alist)
-  (let ((name (cdr (assq 'name alist)))
+\(defun my-navi2ch-article-message-filter-hoge (alist)
+  (let ((number (cdr (assq 'number alist)))
+	(name (cdr (assq 'name alist)))
 	(mail (cdr (assq 'mail alist)))
 	(date (cdr (assq 'date alist)))
 	(message (cdr (assq 'data alist))))
     (if (equal name \"$B$[$2(B\")
 	\"$B$"$\$\!<$s(B\"
-      nil))"
+      nil)))"
   :type '(repeat function)
   :group 'navi2ch-article)
 
 (defcustom navi2ch-article-message-filter-by-name-alist nil
-  "*$B%l%9$r%U%#%k%?$9$kL>A0$H!"CV$-49$($kJ8;zNs$N(B alist$B!#(B
-$BCV$-49$($kJ8;zNs$NBe$o$j$K(B hide $B$d(B important $B$r;XDj$G$-$k!#(B
+  "*$B%l%9$r%U%#%k%?$9$kL>A0$H!"%U%#%k%?=hM}$N(B alist$B!#(B
+
+$BL>A0$K$OJ8;zNs$+!"(B
+\($BJ8;zNs(B $B%7%s%\%k(B)$B$N%j%9%H(B($B3HD%7A<0(B)$B$r;XDj$9$k!#(B
+
+$B3HD%7A<0$r;XDj$9$k$H!"(B
+$B%7%s%\%k$K9g$o$;$F2<5-$NJ}K!$G%l%9$r8!::$9$k!#(B
+
+S,s	$BItJ,0lCW(B
+E,e	$B40A40lCW(B
+F,f	$B$"$$$^$$0lCW(B($B6uGr$H2~9T$NB8:_$rL5;k$7!"(B
+		     $B1Q?t5-9f$NA43Q$HH>3Q$r6hJL$7$J$$ItJ,0lCW(B)
+R,r	$B@55,I=8=(B
+
+$BBgJ8;z$N%7%s%\%k$r;XDj$9$k$HJ8;zNs$NBgJ8;z$H>.J8;z$r6hJL$7!"(B
+$B>.J8;z$N%7%s%\%k$r;XDj$9$k$HJ8;zNs$NBgJ8;z$H>.J8;z$r6hJL$7$J$$!#(B
+
+$B%U%#%k%?=hM}$K$O!"J8;zNs!&%7%s%\%k!&?t;z$N$I$l$+$r;XDj$9$k!#(B
+
+$BJ8;zNs$r;XDj$9$k$H!"%l%9$,$=$NJ8;zNs$KCV$-49$o$k!#(B
+
+$BL>A0$r3HD%7A<0$G;XDj$7$F$$$?>l9g!"(B
+$BCV498e$NJ8;zNsCf$N(B \\1$B!A(B\\9 $B$*$h$S(B \\& $B$O!"0lCW$7$?J8;zNs$KE83+$5$l$k!#(B
+\\1$B!A(B\\9 $B$*$h$S(B \\& $B$N0UL#$K$D$$$F$O!"(B`replace-match'$BEy$r;2>H$N$3$H!#(B
+
+$B%7%s%\%k$r;XDj$9$k$H!"%7%s%\%k$K9g$o$;$F2<5-$N=hM}$,9T$o$l$k!#(B
+
+hide		$B%l%9$r1#$9(B
+important	$B%l%9$r%V%C%/%^!<%/$KEPO?$9$k(B
+
+$B?t;z$r;XDj$9$k$H!"%l%9$NF@E@$K$=$NE@?t$r2C$(!"%U%#%k%?=hM}$rB39T$9$k!#(B
+
 $BNc$($P2<5-$NCM$r@_Dj$9$k$H!"(B
-$BL>A0Ms$K!V$U$,!W$,4^$^$l$F$$$k$H%l%9$,!V$"$\$\!<$s!W$KCV$-49$o$j!"(B
-$BL>A0Ms$K!V%[%2!W$,4^$^$l$F$$$k$H%l%9$,1#$5$l$k!#(B
+$BL>A0$K!V$U$,!W$,4^$^$l$F$$$k$H%l%9$,!V$"$\$\!<$s!W$KCV$-49$o$j!"(B
+$BL>A0$K!V%[%2!W$,4^$^$l$F$$$k$H%l%9$,1#$5$l$k!#(B
 
 '((\"$B$U$,(B\" . \"$B$"$\$\!<$s(B\")
-  (\"$B%[%2(B\" . hide))"
-  :type '(repeat (cons (string :tag "$BL>A0Ms(B")
-		       (string :tag "$BCV498e(B")))
+  ((\"$B%[%2(B\" S) . hide))"
+  :type '(repeat (cons (choice (string :tag "$BL>A0(B")
+			       (choice :tag "($B3HD%7A<0(B)"
+				       (list :tag "$BItJ,0lCW(B"
+					     (string :tag "$BL>A0(B")
+					     (choice :tag "$BBgJ8;z$H>.J8;z$N6hJL(B"
+						     (const :tag "$B$9$k(B"
+							    :value S)
+						     (const :tag "$B$7$J$$(B"
+							    :value s)))
+				       (list :tag "$B40A40lCW(B"
+					     (string :tag "$BL>A0(B")
+					     (choice :tag "$BBgJ8;z$H>.J8;z$N6hJL(B"
+						     (const :tag "$B$9$k(B"
+							    :value E)
+						     (const :tag "$B$7$J$$(B"
+							    :value e)))
+				       (list :tag "$B$"$$$^$$0lCW(B"
+					     (string :tag "$BL>A0(B")
+					     (choice :tag "$BBgJ8;z$H>.J8;z$N6hJL(B"
+						     (const :tag "$B$9$k(B"
+							    :value F)
+						     (const :tag "$B$7$J$$(B"
+							    :value f)))
+				       (list :tag "$B@55,I=8=(B"
+					     (regexp :tag "$BL>A0(B")
+					     (choice :tag "$BBgJ8;z$H>.J8;z$N6hJL(B"
+						     (const :tag "$B$9$k(B"
+							    :value R)
+						     (const :tag "$B$7$J$$(B"
+							    :value r)))))
+		       (choice (string :tag "$BCV$-49$($k(B"
+				       :value "$B$"$\$\!<$s(B")
+			       (const :tag "$B1#$9(B"
+				      :value hide)
+			       (const :tag "$B%V%C%/%^!<%/$KEPO?$9$k(B"
+				      :value important)
+			       (number :tag "$BE@?t$r2C$($k(B"
+				       :value 0))))
   :group 'navi2ch-article)
 
 (defcustom navi2ch-article-message-filter-by-message-alist nil
-  "*$B%l%9$r%U%#%k%?$9$k$?$a$N%l%9K\J8Cf$NJ8;zNs$H!"CV$-49$($kJ8;zNs$N(B alist$B!#(B
-$BCV$-49$($kJ8;zNs$NBe$o$j$K(B hide $B$d(B important $B$r;XDj$G$-$k!#(B
+  "*$B%l%9$r%U%#%k%?$9$k$?$a$N%l%9K\J8$NFbMF$H!"%U%#%k%?=hM}$N(B alist$B!#(B
+
+$B%l%9K\J8$NFbMF$K$OJ8;zNs$+!"(B
+\($BJ8;zNs(B $B%7%s%\%k(B)$B$N%j%9%H(B($B3HD%7A<0(B)$B$r;XDj$9$k!#(B
+
+$B3HD%7A<0$r;XDj$9$k$H!"(B
+$B%7%s%\%k$K9g$o$;$F2<5-$NJ}K!$G%l%9$r8!::$9$k!#(B
+
+S,s	$BItJ,0lCW(B
+E,e	$B40A40lCW(B
+F,f	$B$"$$$^$$0lCW(B($B6uGr$H2~9T$NB8:_$rL5;k$7!"(B
+		     $B1Q?t5-9f$NA43Q$HH>3Q$r6hJL$7$J$$ItJ,0lCW(B)
+R,r	$B@55,I=8=(B
+
+$BBgJ8;z$N%7%s%\%k$r;XDj$9$k$HJ8;zNs$NBgJ8;z$H>.J8;z$r6hJL$7!"(B
+$B>.J8;z$N%7%s%\%k$r;XDj$9$k$HJ8;zNs$NBgJ8;z$H>.J8;z$r6hJL$7$J$$!#(B
+
+$B%U%#%k%?=hM}$K$O!"J8;zNs!&%7%s%\%k!&?t;z$N$I$l$+$r;XDj$9$k!#(B
+
+$BJ8;zNs$r;XDj$9$k$H!"%l%9$,$=$NJ8;zNs$KCV$-49$o$k!#(B
+
+$B%l%9K\J8$NFbMF$r3HD%7A<0$G;XDj$7$F$$$?>l9g!"(B
+$BCV498e$NJ8;zNsCf$N(B \\1$B!A(B\\9 $B$*$h$S(B \\& $B$O!"0lCW$7$?J8;zNs$KE83+$5$l$k!#(B
+\\1$B!A(B\\9 $B$*$h$S(B \\& $B$N0UL#$K$D$$$F$O!"(B`replace-match'$BEy$r;2>H$N$3$H!#(B
+
+$B%7%s%\%k$r;XDj$9$k$H!"%7%s%\%k$K9g$o$;$F2<5-$N=hM}$,9T$o$l$k!#(B
+
+hide		$B%l%9$r1#$9(B
+important	$B%l%9$r%V%C%/%^!<%/$KEPO?$9$k(B
+
+$B?t;z$r;XDj$9$k$H!"%l%9$NF@E@$K$=$NE@?t$r2C$(!"%U%#%k%?=hM}$rB39T$9$k!#(B
+
 $BNc$($P2<5-$NCM$r@_Dj$9$k$H!"(B
-$B%l%9K\J8Cf$K!V$U$,!W$,4^$^$l$F$$$k$H%l%9$,!V$"$\$\!<$s!W$KCV$-49$o$j!"(B
-$B%l%9K\J8Cf$K!V%[%2!W$,4^$^$l$F$$$k$H%l%9$,1#$5$l$k!#(B
+$B%l%9K\J8$K!V$U$,!W$,4^$^$l$F$$$k$H%l%9$,!V$"$\$\!<$s!W$KCV$-49$o$j!"(B
+$B%l%9K\J8$K!V%[%2!W$,4^$^$l$F$$$k$H%l%9$,1#$5$l$k!#(B
 
 '((\"$B$U$,(B\" . \"$B$"$\$\!<$s(B\")
-  (\"$B%[%2(B\" . hide))"
-  :type '(repeat (cons (string :tag "$BK\J8Cf(B")
-		       (string :tag "$BCV498e(B")))
+  ((\"$B%[%2(B\" S) . hide))"
+  :type '(repeat (cons (choice (string :tag "$BK\J8(B")
+			       (choice :tag "($B3HD%7A<0(B)"
+				       (list :tag "$BItJ,0lCW(B"
+					     (string :tag "$BK\J8(B")
+					     (choice :tag "$BBgJ8;z$H>.J8;z$N6hJL(B"
+						     (const :tag "$B$9$k(B"
+							    :value S)
+						     (const :tag "$B$7$J$$(B"
+							    :value s)))
+				       (list :tag "$B40A40lCW(B"
+					     (string :tag "$BK\J8(B")
+					     (choice :tag "$BBgJ8;z$H>.J8;z$N6hJL(B"
+						     (const :tag "$B$9$k(B"
+							    :value E)
+						     (const :tag "$B$7$J$$(B"
+							    :value e)))
+				       (list :tag "$B$"$$$^$$0lCW(B"
+					     (string :tag "$BK\J8(B")
+					     (choice :tag "$BBgJ8;z$H>.J8;z$N6hJL(B"
+						     (const :tag "$B$9$k(B"
+							    :value F)
+						     (const :tag "$B$7$J$$(B"
+							    :value f)))
+				       (list :tag "$B@55,I=8=(B"
+					     (regexp :tag "$BK\J8(B")
+					     (choice :tag "$BBgJ8;z$H>.J8;z$N6hJL(B"
+						     (const :tag "$B$9$k(B"
+							    :value R)
+						     (const :tag "$B$7$J$$(B"
+							    :value r)))))
+		       (choice (string :tag "$BCV$-49$($k(B"
+				       :value "$B$"$\$\!<$s(B")
+			       (const :tag "$B1#$9(B"
+				      :value hide)
+			       (const :tag "$B%V%C%/%^!<%/$KEPO?$9$k(B"
+				      :value important)
+			       (number :tag "$BE@?t$r2C$($k(B"
+				       :value 0))))
   :group 'navi2ch-article)
 
 (defcustom navi2ch-article-message-filter-by-id-alist nil
-  "*$B%l%9$r%U%#%k%?$9$k$?$a$N(B ID $BCf$NJ8;zNs$H!"CV$-49$($kJ8;zNs$N(B alist$B!#(B
-$BCV$-49$($kJ8;zNs$NBe$o$j$K(B hide $B$d(B important $B$r;XDj$G$-$k!#(B
+  "*$B%l%9$r%U%#%k%?$9$k(B ID $B$H!"%U%#%k%?=hM}$N(B alist$B!#(B
+
+ID $B$K$OJ8;zNs$+!"(B
+\($BJ8;zNs(B $B%7%s%\%k(B)$B$N%j%9%H(B($B3HD%7A<0(B)$B$r;XDj$9$k!#(B
+
+$B3HD%7A<0$r;XDj$9$k$H!"(B
+$B%7%s%\%k$K9g$o$;$F2<5-$NJ}K!$G%l%9$r8!::$9$k!#(B
+
+S,s	$BItJ,0lCW(B
+E,e	$B40A40lCW(B
+F,f	$B$"$$$^$$0lCW(B($B6uGr$H2~9T$NB8:_$rL5;k$7!"(B
+		     $B1Q?t5-9f$NA43Q$HH>3Q$r6hJL$7$J$$ItJ,0lCW(B)
+R,r	$B@55,I=8=(B
+
+$BBgJ8;z$N%7%s%\%k$r;XDj$9$k$HJ8;zNs$NBgJ8;z$H>.J8;z$r6hJL$7!"(B
+$B>.J8;z$N%7%s%\%k$r;XDj$9$k$HJ8;zNs$NBgJ8;z$H>.J8;z$r6hJL$7$J$$!#(B
+
+$B%U%#%k%?=hM}$K$O!"J8;zNs!&%7%s%\%k!&?t;z$N$I$l$+$r;XDj$9$k!#(B
+
+$BJ8;zNs$r;XDj$9$k$H!"%l%9$,$=$NJ8;zNs$KCV$-49$o$k!#(B
+
+ID $B$r3HD%7A<0$G;XDj$7$F$$$?>l9g!"(B
+$BCV498e$NJ8;zNsCf$N(B \\1$B!A(B\\9 $B$*$h$S(B \\& $B$O!"0lCW$7$?J8;zNs$KE83+$5$l$k!#(B
+\\1$B!A(B\\9 $B$*$h$S(B \\& $B$N0UL#$K$D$$$F$O!"(B`replace-match'$BEy$r;2>H$N$3$H!#(B
+
+$B%7%s%\%k$r;XDj$9$k$H!"%7%s%\%k$K9g$o$;$F2<5-$N=hM}$,9T$o$l$k!#(B
+
+hide		$B%l%9$r1#$9(B
+important	$B%l%9$r%V%C%/%^!<%/$KEPO?$9$k(B
+
+$B?t;z$r;XDj$9$k$H!"%l%9$NF@E@$K$=$NE@?t$r2C$(!"%U%#%k%?=hM}$rB39T$9$k!#(B
+
 $BNc$($P2<5-$NCM$r@_Dj$9$k$H!"(B
 ID $B$,!V(BFUga1234$B!W$@$H%l%9$,!V$"$\$\!<$s!W$KCV$-49$o$j!"(B
 ID $B$,!V(BhoGE0987$B!W$@$H%l%9$,1#$5$l$k!#(B
 
-'((\"FUga1234\" . \"$B$"$\$\!<$s(B\")
-  (\"hoGE0987\" . hide))"
-  :type '(repeat (cons (string :tag "ID")
-		       (string :tag "$BCV498e(B")))
+'(((\"FUga1234\" E) . \"$B$"$\$\!<$s(B\")
+  ((\"hoGE0987\" E) . hide))"
+  :type '(repeat (cons (choice (string :tag "ID")
+			       (choice :tag "($B3HD%7A<0(B)"
+				       (list :tag "$BItJ,0lCW(B"
+					     (string :tag "ID")
+					     (choice :tag "$BBgJ8;z$H>.J8;z$N6hJL(B"
+						     (const :tag "$B$9$k(B"
+							    :value S)
+						     (const :tag "$B$7$J$$(B"
+							    :value s)))
+				       (list :tag "$B40A40lCW(B"
+					     (string :tag "ID")
+					     (choice :tag "$BBgJ8;z$H>.J8;z$N6hJL(B"
+						     (const :tag "$B$9$k(B"
+							    :value E)
+						     (const :tag "$B$7$J$$(B"
+							    :value e)))
+				       (list :tag "$B$"$$$^$$0lCW(B"
+					     (string :tag "ID")
+					     (choice :tag "$BBgJ8;z$H>.J8;z$N6hJL(B"
+						     (const :tag "$B$9$k(B"
+							    :value F)
+						     (const :tag "$B$7$J$$(B"
+							    :value f)))
+				       (list :tag "$B@55,I=8=(B"
+					     (regexp :tag "ID")
+					     (choice :tag "$BBgJ8;z$H>.J8;z$N6hJL(B"
+						     (const :tag "$B$9$k(B"
+							    :value R)
+						     (const :tag "$B$7$J$$(B"
+							    :value r)))))
+		       (choice (string :tag "$BCV$-49$($k(B"
+				       :value "$B$"$\$\!<$s(B")
+			       (const :tag "$B1#$9(B"
+				      :value hide)
+			       (const :tag "$B%V%C%/%^!<%/$KEPO?$9$k(B"
+				      :value important)
+			       (number :tag "$BE@?t$r2C$($k(B"
+				       :value 0))))
+  :group 'navi2ch-article)
+
+(defcustom navi2ch-article-message-filter-by-mail-alist nil
+  "*$B%l%9$r%U%#%k%?$9$k$?$a$N%a!<%kMs$NFbMF$H!"%U%#%k%?=hM}$N(B alist$B!#(B
+
+$B%a!<%kMs$NFbMF$K$OJ8;zNs$+!"(B
+\($BJ8;zNs(B $B%7%s%\%k(B)$B$N%j%9%H(B($B3HD%7A<0(B)$B$r;XDj$9$k!#(B
+
+$B3HD%7A<0$r;XDj$9$k$H!"(B
+$B%7%s%\%k$K9g$o$;$F2<5-$NJ}K!$G%l%9$r8!::$9$k!#(B
+
+S,s	$BItJ,0lCW(B
+E,e	$B40A40lCW(B
+F,f	$B$"$$$^$$0lCW(B($B6uGr$H2~9T$NB8:_$rL5;k$7!"(B
+		     $B1Q?t5-9f$NA43Q$HH>3Q$r6hJL$7$J$$ItJ,0lCW(B)
+R,r	$B@55,I=8=(B
+
+$BBgJ8;z$N%7%s%\%k$r;XDj$9$k$HJ8;zNs$NBgJ8;z$H>.J8;z$r6hJL$7!"(B
+$B>.J8;z$N%7%s%\%k$r;XDj$9$k$HJ8;zNs$NBgJ8;z$H>.J8;z$r6hJL$7$J$$!#(B
+
+$B%U%#%k%?=hM}$K$O!"J8;zNs!&%7%s%\%k!&?t;z$N$I$l$+$r;XDj$9$k!#(B
+
+$BJ8;zNs$r;XDj$9$k$H!"%l%9$,$=$NJ8;zNs$KCV$-49$o$k!#(B
+
+$B%a!<%kMs$NFbMF$r3HD%7A<0$G;XDj$7$F$$$?>l9g!"(B
+$BCV498e$NJ8;zNsCf$N(B \\1$B!A(B\\9 $B$*$h$S(B \\& $B$O!"0lCW$7$?J8;zNs$KE83+$5$l$k!#(B
+\\1$B!A(B\\9 $B$*$h$S(B \\& $B$N0UL#$K$D$$$F$O!"(B`replace-match'$BEy$r;2>H$N$3$H!#(B
+
+$B%7%s%\%k$r;XDj$9$k$H!"%7%s%\%k$K9g$o$;$F2<5-$N=hM}$,9T$o$l$k!#(B
+
+hide		$B%l%9$r1#$9(B
+important	$B%l%9$r%V%C%/%^!<%/$KEPO?$9$k(B
+
+$B?t;z$r;XDj$9$k$H!"%l%9$NF@E@$K$=$NE@?t$r2C$(!"%U%#%k%?=hM}$rB39T$9$k!#(B
+
+$BNc$($P2<5-$NCM$r@_Dj$9$k$H!"(B
+$B%a!<%kMs$K!V$U$,!W$,4^$^$l$F$$$k$H%l%9$,!V$"$\$\!<$s!W$KCV$-49$o$j!"(B
+$B%a!<%kMs$K!V%[%2!W$,4^$^$l$F$$$k$H%l%9$,1#$5$l$k!#(B
+
+'((\"$B$U$,(B\" . \"$B$"$\$\!<$s(B\")
+  ((\"$B%[%2(B\" S) . hide))"
+  :type '(repeat (cons (choice (string :tag "$B%a!<%kMs(B")
+			       (choice :tag "($B3HD%7A<0(B)"
+				       (list :tag "$BItJ,0lCW(B"
+					     (string :tag "$B%a!<%kMs(B")
+					     (choice :tag "$BBgJ8;z$H>.J8;z$N6hJL(B"
+						     (const :tag "$B$9$k(B"
+							    :value S)
+						     (const :tag "$B$7$J$$(B"
+							    :value s)))
+				       (list :tag "$B40A40lCW(B"
+					     (string :tag "$B%a!<%kMs(B")
+					     (choice :tag "$BBgJ8;z$H>.J8;z$N6hJL(B"
+						     (const :tag "$B$9$k(B"
+							    :value E)
+						     (const :tag "$B$7$J$$(B"
+							    :value e)))
+				       (list :tag "$B$"$$$^$$0lCW(B"
+					     (string :tag "$B%a!<%kMs(B")
+					     (choice :tag "$BBgJ8;z$H>.J8;z$N6hJL(B"
+						     (const :tag "$B$9$k(B"
+							    :value F)
+						     (const :tag "$B$7$J$$(B"
+							    :value f)))
+				       (list :tag "$B@55,I=8=(B"
+					     (regexp :tag "$B%a!<%kMs(B")
+					     (choice :tag "$BBgJ8;z$H>.J8;z$N6hJL(B"
+						     (const :tag "$B$9$k(B"
+							    :value R)
+						     (const :tag "$B$7$J$$(B"
+							    :value r)))))
+		       (choice (string :tag "$BCV$-49$($k(B"
+				       :value "$B$"$\$\!<$s(B")
+			       (const :tag "$B1#$9(B"
+				      :value hide)
+			       (const :tag "$B%V%C%/%^!<%/$KEPO?$9$k(B"
+				      :value important)
+			       (number :tag "$BE@?t$r2C$($k(B"
+				       :value 0))))
+  :group 'navi2ch-article)
+
+(defcustom navi2ch-article-message-filter-by-subject-alist nil
+  "*$B%l%9$r%U%#%k%?$9$k$?$a$N%9%l$N%?%$%H%k$H!"%U%#%k%?=hM}$N(B alist$B!#(B
+
+$B%9%l$N%?%$%H%k$K$OJ8;zNs$+!"(B
+\($BJ8;zNs(B $B%7%s%\%k(B)$B$N%j%9%H(B($B3HD%7A<0(B)$B$r;XDj$9$k!#(B
+
+$B3HD%7A<0$r;XDj$9$k$H!"(B
+$B%7%s%\%k$K9g$o$;$F2<5-$NJ}K!$G%l%9$r8!::$9$k!#(B
+
+S,s	$BItJ,0lCW(B
+E,e	$B40A40lCW(B
+F,f	$B$"$$$^$$0lCW(B($B6uGr$H2~9T$NB8:_$rL5;k$7!"(B
+		     $B1Q?t5-9f$NA43Q$HH>3Q$r6hJL$7$J$$ItJ,0lCW(B)
+R,r	$B@55,I=8=(B
+
+$BBgJ8;z$N%7%s%\%k$r;XDj$9$k$HJ8;zNs$NBgJ8;z$H>.J8;z$r6hJL$7!"(B
+$B>.J8;z$N%7%s%\%k$r;XDj$9$k$HJ8;zNs$NBgJ8;z$H>.J8;z$r6hJL$7$J$$!#(B
+
+$B%U%#%k%?=hM}$K$O!"J8;zNs!&%7%s%\%k!&?t;z$N$I$l$+$r;XDj$9$k!#(B
+
+$BJ8;zNs$r;XDj$9$k$H!"%l%9$,$=$NJ8;zNs$KCV$-49$o$k!#(B
+
+$B%9%l$N%?%$%H%k$r3HD%7A<0$G;XDj$7$F$$$?>l9g!"(B
+$BCV498e$NJ8;zNsCf$N(B \\1$B!A(B\\9 $B$*$h$S(B \\& $B$O!"0lCW$7$?J8;zNs$KE83+$5$l$k!#(B
+\\1$B!A(B\\9 $B$*$h$S(B \\& $B$N0UL#$K$D$$$F$O!"(B`replace-match'$BEy$r;2>H$N$3$H!#(B
+
+$B%7%s%\%k$r;XDj$9$k$H!"%7%s%\%k$K9g$o$;$F2<5-$N=hM}$,9T$o$l$k!#(B
+
+hide		$B%l%9$r1#$9(B
+important	$B%l%9$r%V%C%/%^!<%/$KEPO?$9$k(B
+
+$B?t;z$r;XDj$9$k$H!"%l%9$NF@E@$K$=$NE@?t$r2C$(!"%U%#%k%?=hM}$rB39T$9$k!#(B
+
+$BNc$($P2<5-$NCM$r@_Dj$9$k$H!"(B
+$B%9%l$N%?%$%H%k$K!V$U$,!W$,4^$^$l$F$$$k$H%l%9$,!V$"$\$\!<$s!W$KCV$-49$o$j!"(B
+$B%9%l$N%?%$%H%k$K!V%[%2!W$,4^$^$l$F$$$k$H%l%9$,1#$5$l$k!#(B
+
+'((\"$B$U$,(B\" . \"$B$"$\$\!<$s(B\")
+  ((\"$B%[%2(B\" S) . hide))"
+  :type '(repeat (cons (choice (string :tag "$B%9%l$N%?%$%H%k(B")
+			       (choice :tag "($B3HD%7A<0(B)"
+				       (list :tag "$BItJ,0lCW(B"
+					     (string :tag "$B%9%l$N%?%$%H%k(B")
+					     (choice :tag "$BBgJ8;z$H>.J8;z$N6hJL(B"
+						     (const :tag "$B$9$k(B"
+							    :value S)
+						     (const :tag "$B$7$J$$(B"
+							    :value s)))
+				       (list :tag "$B40A40lCW(B"
+					     (string :tag "$B%9%l$N%?%$%H%k(B")
+					     (choice :tag "$BBgJ8;z$H>.J8;z$N6hJL(B"
+						     (const :tag "$B$9$k(B"
+							    :value E)
+						     (const :tag "$B$7$J$$(B"
+							    :value e)))
+				       (list :tag "$B$"$$$^$$0lCW(B"
+					     (string :tag "$B%9%l$N%?%$%H%k(B")
+					     (choice :tag "$BBgJ8;z$H>.J8;z$N6hJL(B"
+						     (const :tag "$B$9$k(B"
+							    :value F)
+						     (const :tag "$B$7$J$$(B"
+							    :value f)))
+				       (list :tag "$B@55,I=8=(B"
+					     (regexp :tag "$B%9%l$N%?%$%H%k(B")
+					     (choice :tag "$BBgJ8;z$H>.J8;z$N6hJL(B"
+						     (const :tag "$B$9$k(B"
+							    :value R)
+						     (const :tag "$B$7$J$$(B"
+							    :value r)))))
+		       (choice (string :tag "$BCV$-49$($k(B"
+				       :value "$B$"$\$\!<$s(B")
+			       (const :tag "$B1#$9(B"
+				      :value hide)
+			       (const :tag "$B%V%C%/%^!<%/$KEPO?$9$k(B"
+				      :value important)
+			       (number :tag "$BE@?t$r2C$($k(B"
+				       :value 0))))
+  :group 'navi2ch-article)
+
+(defcustom navi2ch-article-auto-activate-message-filter t
+  "*non-nil $B$J$i!"%U%#%k%?5!G=$r%G%U%)%k%H$G(B on $B$K$9$k!#(B"
+  :type 'boolean
+  :group 'navi2ch-article)
+
+(defcustom navi2ch-article-use-message-filter-cache t
+  "*non-nil $B$J$i!"%U%#%k%?=hM}$G%-%c%C%7%e$rMxMQ$9$k!#(B
+$B%-%c%C%7%e$O!"%U%#%k%?$N%"%s%I%%>pJs$NJ];}$b7s$M$k!#(B"
+  :type 'boolean
+  :group 'navi2ch-article)
+
+(defcustom navi2ch-article-sort-message-filter-rules nil
+  "*non-nil $B$J$i!">r7o$,0lCW$7$?%U%#%k%?9`L\$r(B alist $B$N@hF,$K;}$C$F$/$k!#(B
+
+$BNc$($P!"(B`navi2ch-article-message-filter-by-name-alist'$B$r(B
+$B2<5-$NCM$K@_Dj$7$F$$$F!V%[%2!W$H$$$&L>A0Ms$N%l%9$KEv$?$C$?>l9g!"(B
+
+'((\"$B$U$,(B\" . \"$B$"$\$\!<$s(B\")
+  ((\"$B%[%2(B\" S) . hide))
+
+`navi2ch-article-message-filter-by-name-alist'$B$NCM$O(B
+$B>r7o$,0lCW$7$?(B '((\"$B%[%2(B\" S) . hide) $B$,@hF,$KMh$k$h$&$KJB$S49$($i$l!"(B
+$B<!$N$h$&$KJQ99$5$l$k!#(B
+
+'(((\"$B%[%2(B\" S) . hide)
+  (\"$B$U$,(B\" . \"$B$"$\$\!<$s(B\"))"
+  :type 'boolean
+  :group 'navi2ch-article)
+
+(defcustom navi2ch-article-message-replace-below nil
+  "*$B%U%#%k%?$K$h$C$F%l%9$rCV$-49$($k$?$a$NF@E@$N$7$-$$CM$H!"(B
+$BCV$-49$($kJ8;zNs!#(B
+$BF@E@$,$3$NCM$h$j>.$5$$$H%l%9$,CV$-49$o$k!#(B
+
+$BNc$($P2<5-$NCM$r@_Dj$9$k$H!"(B
+$B%l%9$NF@E@$,(B-1000$B$h$j>.$5$$$H%l%9$,!V$"$\$\!<$s!W$KCV$-49$o$k!#(B
+
+'(-1000 . \"$B$"$\$\!<$s(B\")"
+  :type '(choice (const :tag "off"
+			:value nil)
+		 (cons :tag "$B@_Dj$9$k(B"
+		       (number :tag "$B$7$-$$CM(B"
+			       :value 0)
+		       (string :tag "$BCV498e(B"
+			       :value "$B$"$\$\!<$s(B")))
+  :group 'navi2ch-article)
+
+(defcustom navi2ch-article-message-hide-below nil
+  "*$B%U%#%k%?$K$h$C$F%l%9$r1#$9$?$a$NF@E@$N$7$-$$CM!#(B
+$BF@E@$,$3$NCM$h$j>.$5$$$H%l%9$,1#$5$l$k!#(B"
+  :type '(choice (const :tag "off"
+			:value nil)
+		 (number :tag "$B$7$-$$CM(B"
+			 :value 0))
+  :group 'navi2ch-article)
+
+(defcustom navi2ch-article-message-add-important-above nil
+  "*$B%U%#%k%?$K$h$C$F%l%9$r%V%C%/%^!<%/$KEPO?$9$k$?$a$NF@E@$N$7$-$$CM!#(B
+$BF@E@$,$3$NCM$h$jBg$-$$$H%l%9$,%V%C%/%^!<%/$KEPO?$5$l$k!#(B"
+  :type '(choice (const :tag "off"
+			:value nil)
+		 (number :tag "$B$7$-$$CM(B"
+			 :value 0))
   :group 'navi2ch-article)
 
 (defcustom navi2ch-article-save-info-wrapper-func nil
@@ -732,10 +1249,7 @@ ID $B$,!V(BhoGE0987$B!W$@$H%l%9$,1#$5$l$k!#(B
   :group 'navi2ch-article)
 
 ;;; message variables
-(defcustom navi2ch-message-user-name
-  (cond ((featurep 'xemacs) "$BL>L5$7$5$s!w#X#E#m#a#c#s(B")
-	((featurep 'meadow) "$BL>L5$7$5$s!w#M#e#a#d#o#w(B")
-	(t "$BL>L5$7$5$s!w#E#m#a#c#s(B"))
+(defcustom navi2ch-message-user-name ""
   "*$B%G%U%)%k%H$NL>A0!#(B"
   :type 'string
   :group 'navi2ch-message)
@@ -755,6 +1269,17 @@ ID $B$,!V(BhoGE0987$B!W$@$H%l%9$,1#$5$l$k!#(B
   :type 'string
   :group 'navi2ch-message)
 
+(defcustom navi2ch-message-mail-address-alist nil
+  "*$BHD$4$H$N%G%U%)%k%H$N%a!<%k%"%I%l%9$N(B alist$B!#(B
+
+$B$?$H$($P<!$N$h$&$K@_Dj$7$F$*$/$H!"(B
+$B%K%e!<%9B.JsHD$G$O(B \"someone@example.com\"$B!"(BUNIX $BHD$G$O(B \"sage\" $B$,(B
+$B%G%U%)%k%H$N%a!<%k%"%I%l%9$K$J$k!#(B
+  '((\"news\" . \"someone@example.com\")
+    (\"unix\" . \"sage\"))"
+  :type '(repeat (cons (string :tag "$BHD(B  ") (string :tag "$BL>A0(B")))
+  :group 'navi2ch-message)
+
 (defcustom navi2ch-message-ask-before-write nil
   "*non-nil $B$J$i!"%l%9$r=q$-;O$a$k$H$-$K3NG'%a%C%;!<%8$rI=<($9$k!#(B"
   :type '(choice (const :tag "yes-or-no-p $B$G3NG'(B" yes-or-no-p)
@@ -897,6 +1422,18 @@ Navi2ch$B%+%F%4%j$K!VAw?.95$(!WHD$,<+F0E*$KDI2C$5$l$^$9!#(B
   :type 'string
   :group 'navi2ch-message)
 
+(defcustom navi2ch-message-sendlog-response-limit 1000
+  "*$BAw?.95$((B 1$B%9%l$"$?$j$N%l%9?t$N>e8B!#(Bnil $B$J$i$P@)8B$7$J$$!#(B"
+  :type '(choice (integer :tag "$B%l%9?t$N>e8B(B")
+		 (const :tag "$BL5@)8B(B" nil))
+  :group 'navi2ch-message)
+
+(defcustom navi2ch-message-sendlog-volume-format "%s (Part %s)"
+  "*$BAw?.95$(%9%l$rJ,3d$9$k$H$-$N%U%)!<%^%C%H!#(B
+$B:G=i$N(B %s $B$,%9%l$N%?%$%H%k$G!"(B2$BHVL\$N(B %s $B$,%9%l$NHV9f$GCV$-49$($i$l$k!#(B"
+  :type 'string
+  :group 'navi2ch-message)
+
 ;; net variables
 (defcustom navi2ch-net-http-proxy
   (if (string= (getenv "HTTP_PROXY") "")
@@ -908,13 +1445,13 @@ Navi2ch$B%+%F%4%j$K!VAw?.95$(!WHD$,<+F0E*$KDI2C$5$l$^$9!#(B
   :group 'navi2ch-net)
 
 (defcustom navi2ch-net-http-proxy-userid nil
-  "$B%W%m%-%7G'>Z$K;H$&%f!<%6L>!#(B"
+  "*$B%W%m%-%7G'>Z$K;H$&%f!<%6L>!#(B"
   :type '(choice (string :tag "$B%f!<%6L>$r;XDj(B")
 		 (const :tag "$B%f!<%6L>$r;H$o$J$$(B" nil))
   :group 'navi2ch-net)
 
 (defcustom navi2ch-net-http-proxy-password nil
-  "$B%W%m%-%7G'>Z$K;H$&%Q%9%o!<%I!#(B"
+  "*$B%W%m%-%7G'>Z$K;H$&%Q%9%o!<%I!#(B"
   :type '(choice (string :tag "$B%Q%9%o!<%I$r;XDj(B")
 		 (const :tag "$B%Q%9%o!<%I$r;H$o$J$$(B" nil))
   :group 'navi2ch-net)
@@ -926,6 +1463,28 @@ Navi2ch$B%+%F%4%j$K!VAw?.95$(!WHD$,<+F0E*$KDI2C$5$l$^$9!#(B
   :type 'boolean
   :group 'navi2ch-net)
 
+(defcustom navi2ch-net-http-proxy-for-send-message nil
+  "*$B%l%9=q$-9~$_$K;H$&(B HTTP $B%W%m%-%7$N(B URL$B!#(B
+nil $B$N$H$-$O%W%m%-%7$H$7$F(B `navi2ch-net-http-proxy' $B$,;H$o$l$k!#(B
+`navi2ch-net-send-message-use-http-proxy' $B$,(B non-nil $B$N$H$-$N$_M-8z!#(B"
+  :type '(choice (string :tag "$B%W%m%-%7$r;XDj(B")
+		 (const :tag "`navi2ch-net-http-proxy' $B$HF1$8(B" nil))
+  :group 'navi2ch-net)
+
+(defcustom navi2ch-net-http-proxy-userid-for-send-message nil
+  "*$B%l%9=q$-9~$_;~$K%W%m%-%7G'>Z$K;H$&%f!<%6L>!#(B
+`navi2ch-net-http-proxy-for-send-message' $B$,(B non-nil $B$N$H$-$N$_M-8z!#(B"
+  :type '(choice (string :tag "$B%f!<%6L>$r;XDj(B")
+		 (const :tag "$B%f!<%6L>$r;H$o$J$$(B" nil))
+  :group 'navi2ch-net)
+
+(defcustom navi2ch-net-http-proxy-password-for-send-message nil
+  "*$B%l%9=q$-9~$_;~$K%W%m%-%7G'>Z$K;H$&%Q%9%o!<%I!#(B
+`navi2ch-net-http-proxy-for-send-message' $B$,(B non-nil $B$N$H$-$N$_M-8z!#(B"
+  :type '(choice (string :tag "$B%Q%9%o!<%I$r;XDj(B")
+		 (const :tag "$B%Q%9%o!<%I$r;H$o$J$$(B" nil))
+  :group 'navi2ch-net)
+
 (defcustom navi2ch-net-force-update nil
   "*non-nil $B$J$i!"%U%!%$%k$r<hF@$9$k$^$($K99?7$NM-L5$r3NG'$7$J$/$J$k!#(B
 nil $B$J$i!"99?7$5$l$F$$$J$$%U%!%$%k$NITI,MW$JE>Aw$O$7$J$$!#(B"
@@ -988,7 +1547,7 @@ ask $B$J$iJ]B8$9$kA0$K<ALd$9$k(B
   :type 'string
   :group 'navi2ch)
 
-(defcustom navi2ch-auto-update t
+(defcustom navi2ch-auto-update nil
   "*non-nil $B$J$i!"5/F0;~$K(B `navi2ch-update-file' $B$r99?7$7$F<B9T$9$k!#(B
 $B%U%!%$%k$,<B9T$5$l$k$N$O!"(B
  - `navi2ch-update-file' $B$,99?7$5$l$F$$$F!"(B
@@ -1000,6 +1559,44 @@ ask $B$J$iJ]B8$9$kA0$K<ALd$9$k(B
   :type 'boolean
   :group 'navi2ch)
 
+;;; auto modify variables
+(defcustom navi2ch-auto-modify-file
+  (let ((file (or (locate-library navi2ch-init-file)
+		  (and (file-name-absolute-p navi2ch-init-file)
+		       (expand-file-name navi2ch-init-file)))))
+    (when (and file
+	       (not (string-match "\\.elc\\(\\.\\(Z\\|gz\\|bz2\\)\\)?\\'"
+				  file)))
+      file))
+  "*$B@_Dj$r<+F0E*$KJQ99$7$FJ]B8$9$k%U%!%$%k!#(B
+nil $B$J$i!"(B`customize'$B$rMxMQ$7$F(B`custom-file'$B$KJ]B8$9$k!#(B
+
+$B$3$N%U%!%$%kC1BN$,<+F0E*$K%m!<%I$5$l$k$3$H$O$J$$$N$G!"(B
+`navi2ch-init-file'$B0J30$N%U%!%$%k$r;XDj$7$?>l9g(B
+\(`navi2ch-init-file'$B$r(B byte-compile $B$7$?>l9g$r4^$`(B)$B$OI,MW$K1~$8$F!"(B
+
+\(load navi2ch-auto-modify-file)
+
+$B$r(B`navi2ch-init-file'$B$KDI2C$9$k$J$I$NJ}K!$GL@<(E*$K%U%!%$%k$r(B
+$B%m!<%I$9$k$3$H!#(B"
+  :type '(choice (file :tag "$B%U%!%$%k(B")
+		 (const :tag "custom-file" nil))
+  :group 'navi2ch)
+
+(defcustom navi2ch-auto-modify-truncate-list-alist nil
+  "*$B%j%9%H7?JQ?t$rJ]B8$9$k$H$-$N!"JQ?tL>$H$=$N:GBgMWAG?t$N(B alist$B!#(B
+
+$BNc$($P2<5-$NCM$r@_Dj$9$k$H!"(B
+`navi2ch-article-message-filter-by-id-alist'$B$H(B
+`navi2ch-article-message-filter-by-message-alist'$B$NMWAG$O!"(B
+$B<+F0JQ99!&J]B8$N:]$K$=$l$>$l(B10$B8D0J2<!&(B100$B8D0J2<$K@Z$j5M$a$i$l$k!#(B
+
+'((navi2ch-article-message-filter-by-id-alist . 10)
+  (navi2ch-article-message-filter-by-message-alist . 100))"
+  :type '(repeat (cons (variable :tag "$BJQ?tL>(B")
+		       (integer :tag "$B:GBgMWAG?t(B")))
+  :group 'navi2ch)
+
 (defcustom navi2ch-icon-directory
   (cond ((fboundp 'locate-data-directory)
 	 (locate-data-directory "navi2ch"))
@@ -1074,16 +1671,19 @@ a symbol `bitmap', `xbm' or `xpm' in order to force the image format."
 (defvar navi2ch-list-mode-hook nil)
 (defvar navi2ch-list-exit-hook nil)
 (defvar navi2ch-list-after-sync-hook nil)
+(defvar navi2ch-list-get-category-list-hook nil)
 (defvar navi2ch-board-mode-hook nil)
 (defvar navi2ch-board-exit-hook nil)
 (defvar navi2ch-board-before-sync-hook nil)
 (defvar navi2ch-board-after-sync-hook nil)
 (defvar navi2ch-board-select-board-hook nil)
+(defvar navi2ch-board-get-subject-list-hook nil)
 (defvar navi2ch-article-mode-hook nil)
 (defvar navi2ch-article-exit-hook nil)
 (defvar navi2ch-article-before-sync-hook nil)
 (defvar navi2ch-article-after-sync-hook nil)
 (defvar navi2ch-article-arrange-message-hook nil)
+(defvar navi2ch-article-get-message-list-hook nil)
 (defvar navi2ch-bookmark-mode-hook nil)
 (defvar navi2ch-bookmark-exit-hook nil)
 (defvar navi2ch-articles-mode-hook nil)
@@ -1109,6 +1709,7 @@ a symbol `bitmap', `xbm' or `xpm' in order to force the image format."
 (defvar navi2ch-mona-undo-setup-hook nil)
 (defvar navi2ch-directory-mode-hook nil)
 (defvar navi2ch-directory-exit-hook nil)
+(defvar navi2ch-auto-modify-save-hook nil)
 
 ;; load hooks
 (defvar navi2ch-article-load-hook nil)
@@ -1148,6 +1749,8 @@ a symbol `bitmap', `xbm' or `xpm' in order to force the image format."
     ;; (define-key map "\C-c2" 'navi2ch-two-pane)
     ;; (define-key map "\C-c3" 'navi2ch-three-pane)
     (define-key map "\C-c\C-o" 'navi2ch-message-jump-to-message-buffer)
+    (define-key map "\C-c\C-n" 'navi2ch-article-forward-sticky-buffer)
+    (define-key map "\C-c\C-p" 'navi2ch-article-backward-sticky-buffer)
     (setq navi2ch-global-map map)))
 
 (defvar navi2ch-global-view-map nil
