@@ -1,6 +1,6 @@
 ;;; navi2ch.el --- Navigator for 2ch for Emacsen
 
-;; Copyright (C) 2000-2002 by Navi2ch Project
+;; Copyright (C) 2000-2003 by Navi2ch Project
 
 ;; Author: Taiki SUGAWARA <taiki@users.sourceforge.net>
 ;; Keywords: network, 2ch
@@ -23,7 +23,7 @@
 ;;; Code:
 (provide 'navi2ch)
 (defvar navi2ch-ident
-  "$Id: navi2ch.el,v 1.53 2002/10/19 13:52:52 tabmore Exp $")
+  "$Id: navi2ch.el,v 1.62 2003/09/05 22:15:16 mizmiz Exp $")
 
 (eval-when-compile (require 'cl))
 
@@ -43,17 +43,21 @@
 (require 'navi2ch-search)
 (require 'navi2ch-directory)
 (require 'navi2ch-message)
-(and navi2ch-on-emacs21
-     (require 'navi2ch-e21))
+(navi2ch-ifxemacs
+    (require 'navi2ch-xmas)
+  (when navi2ch-on-emacs21
+    (require 'navi2ch-e21)))
 (require 'navi2ch-splash)
 (require 'navi2ch-version)
 (require 'navi2ch-jbbs-net)
 (require 'navi2ch-jbbs-shitaraba)
 (require 'navi2ch-machibbs)
+(require 'navi2ch-futaba)
 (require 'navi2ch-multibbs)
 (require 'navi2ch-mona)
 (require 'navi2ch-oyster)
 (require 'navi2ch-localfile)
+(require 'navi2ch-auto-modify)
 
 (defgroup navi2ch nil
   "Navigator for 2ch."
@@ -67,8 +71,7 @@
 (defun navi2ch-kill-emacs-hook ()
   (run-hooks 'navi2ch-kill-emacs-hook)
   (navi2ch-save-status)
-  (if navi2ch-use-lock
-      (navi2ch-unlock)))
+  (navi2ch-unlock))
 
 ;;;###autoload
 (defun navi2ch (&optional arg)
@@ -80,10 +83,13 @@
     (when (file-exists-p navi2ch-update-file)
       (load-file navi2ch-update-file))
     (load navi2ch-init-file t)
-    (if navi2ch-use-lock
-	(navi2ch-lock))
+    (navi2ch-lock)
     (if navi2ch-auto-update
-	(navi2ch-update))
+	(condition-case err
+	    (navi2ch-update)
+	  ((error quit)
+	   (navi2ch-unlock)
+	   (signal (car err) (cdr err)))))
     (add-hook 'kill-emacs-hook 'navi2ch-kill-emacs-hook)
     (run-hooks 'navi2ch-load-status-hook)
     (run-hooks 'navi2ch-hook)
@@ -135,8 +141,7 @@ SUSPEND $B$,(B non-nil $B$J$i(B buffer $B$r>C$5$J$$(B"
           (kill-buffer x))))
     (unless suspend
       (setq navi2ch-init nil)
-      (if navi2ch-use-lock
-	  (navi2ch-unlock))
+      (navi2ch-unlock)
       (remove-hook 'kill-emacs-hook 'navi2ch-kill-emacs-hook))))
 
 (defun navi2ch-suspend ()
@@ -280,13 +285,22 @@ BACKUP $B$,(B non-nil $B$N>l9g$O85$N%U%!%$%k$r%P%C%/%"%C%W$9$k!#(B"
 	    (setq temp-file (navi2ch-make-temp-file
 			     (file-name-directory file)))
 	    (with-temp-file temp-file
-	      (let ((standard-output (current-buffer)))
+	      (let ((standard-output (current-buffer))
+		    print-length print-level)
 		(prin1 info)))
 	    (if (and backup (file-exists-p file))
 		(rename-file file backup-file t))
 	    ;; $B>e$N(B rename $B$,@.8y$7$F2<$,<:GT$7$F$b!"(Bnavi2ch-load-info
 	    ;; $B$,%P%C%/%"%C%W%U%!%$%k$+$iFI$s$G$/$l$k!#(B
-	    (rename-file temp-file file t))
+	    (condition-case err
+		(progn
+		  (unless (file-exists-p file)
+		    (with-temp-file file)) ; $B%U%!%$%k$,:n@.2DG=$+%A%'%C%/(B
+		  (rename-file temp-file file t))
+	      (file-error
+	       (message "%s: %s"
+			(cadr err)
+			(mapconcat #'identity (cddr err) ", ")))))
 	(if (and temp-file (file-exists-p temp-file))
 	    (delete-file temp-file))))))
 
@@ -446,24 +460,15 @@ CHANGED-LIST $B$K$D$$$F$O(B `navi2ch-list-get-changed-status' $B$r;2>H!#(B"
 
 (defun navi2ch-lock ()
   "`navi2ch-directory' $B$r%m%C%/$9$k!#(B"
-  (let ((redo t)
-	error-message)
-    (while redo
-      (setq redo nil)
-      (unless (navi2ch-lock-directory navi2ch-directory navi2ch-lock-name)
-	(setq error-message "$B%G%#%l%/%H%j$N%m%C%/$K<:GT$7$^$7$?!#(B")
-	(cond ((y-or-n-p (format "%s$B$b$&0lEY;n$7$^$9$+(B? "
-				 error-message))
-	       (setq redo t))
-	      ((yes-or-no-p (format "%s$B4m81$r>5CN$GB3$1$^$9$+(B? "
-				    error-message))
-	       nil)
-	      (t
-	       (error "lock failed")))))))
+  (if (and navi2ch-use-lock
+	   (not (navi2ch-lock-directory navi2ch-directory navi2ch-lock-name))
+	   (not (yes-or-no-p "$B%G%#%l%/%H%j$N%m%C%/$K<:GT$7$^$7$?!#4m81$r>5CN$GB3$1$^$9$+(B?")))
+      (error "lock failed")))
 
 (defun navi2ch-unlock ()
   "`navi2ch-directory' $B$N%m%C%/$r2r=|$9$k!#(B"
-  (navi2ch-unlock-directory navi2ch-directory navi2ch-lock-name))
+  (if navi2ch-use-lock
+      (navi2ch-unlock-directory navi2ch-directory navi2ch-lock-name)))
 
 (defun navi2ch-ident-list ()
   "$B%m!<%I$7$F$$$k(B Navi2ch $B$N3F%b%8%e!<%k$N(B Id $B$rI=<($9$k!#(B"
@@ -497,5 +502,13 @@ CHANGED-LIST $B$K$D$$$F$O(B `navi2ch-list-get-changed-status' $B$r;2>H!#(B"
     (message "falling back...")
     (apply 'navi2ch-browse-url-internal url args)))
 
+(eval-when-compile
+  (mapatoms (lambda (symbol)
+	      (if (and (fboundp symbol)
+		       (string-match "\\`navi2ch-" (symbol-name symbol))
+		       (eq (get symbol 'byte-optimizer)
+			   'byte-compile-inline-expand))
+		  (byte-compile symbol)))))
+
 (run-hooks 'navi2ch-load-hook)
 ;;; navi2ch.el ends here
